/*
 * ExtensionMatcher.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using System.Collections;
using System.Collections.Specialized;
using System.IO;
using TopCoder.File.Statistics;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics.Matcher
{
    /// <summary>
    /// Matches a filename with one or more file extension.
    /// </summary>
    ///
    /// <remarks>
    /// The file extension is matched in a case-insensitive manner
    /// against a list of extensions.
    /// It is possible to instantiate this class using reflection
    /// and has its extensions list loaded from a configuration file.
    /// The constructor with nameSpace and prefix arguments is used for this purpose.
    /// There should be a multi-value property with the name <c>prefix + "extensions"</c>,
    /// which contains the extensions list.
    /// Although the property is required, empty is allowed to match a file with no extension.
    /// </remarks>
    ///
    /// <threadsafety static="true" instance="true"/>
    public class ExtensionMatcher : FileNameMatcher
    {
        /// <summary>
        /// The configuration key that contains multiple file extensions to match.
        /// </summary>
        ///
        /// <remarks>
        /// The actual property key name is <c>prefix + "extensions"</c>.
        /// The matching is carried out in a case-insensitive manner.
        /// The property is required, but empty is allowed in order to match
        /// a file with no extension.
        /// </remarks>
        public static readonly string ExtensionsPropertyKey = "extensions";

        /// <summary>
        /// A list of lower-case file extensions (String) to match.
        /// This list is initialized in constructor and is immutable.
        /// Developer notes. Although the design recommends IList, it's
        /// better to use IDictionary; because IList.Contains is O(n),
        /// while IDictionary.Contains can be O(1).
        /// It's currently implemented with a case-insensitive string HashTable
        /// created via CollectionsUtil.CreateCaseInsensitiveHashtable.
        /// </summary>
        private IDictionary extensions;

        /// <summary>
        /// Initializes a new instance of the <see cref="ExtensionMatcher"/>
        /// with the specified single extension.
        /// </summary>
        ///
        /// <param name="extension">
        /// A single file extension to match. It can be empty.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="extension"/> is a null reference.
        /// </exception>
        public ExtensionMatcher(string extension)
        {
            // cannot be null
            if (extension == null)
            {
                throw new ArgumentNullException("extension");
            }

            // save to instance members (case-insensitive hash table)
            this.extensions = CollectionsUtil.CreateCaseInsensitiveHashtable();
            this.extensions.Add(extension, null);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ExtensionMatcher"/>
        /// with the specified list of extensions.
        /// </summary>
        ///
        /// <param name="extensions">
        /// The list of extensions (string) to match.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="extensions"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="extensions"/> is empty, contains non-string instance,
        /// or contains duplicate element.
        /// </exception>
        public ExtensionMatcher(IList extensions)
        {
            // cannot be null
            if (extensions == null)
            {
                throw new ArgumentNullException("extensions");
            }
            // cannot be empty
            if (extensions.Count == 0)
            {
                throw new ArgumentException("'extensions' cannot be an empty list", "extensions");
            }
            // cannot contain non-string instance
            foreach (object obj in extensions)
            {
                if (!(obj is string))
                {
                    throw new ArgumentException(
                        "'extensions' contains a non-string instance", "extensions");
                }
            }

            // add list to case-insensitive hashtable (duplicate will throw ArgumentException)
            this.extensions = CollectionsUtil.CreateCaseInsensitiveHashtable();
            Helper.AddListToDictionary(extensions, this.extensions);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ExtensionMatcher"/>
        /// which loads the extensions list from the specified nameSpace
        /// and prefix of the configuration.
        /// </summary>
        ///
        /// <remarks>
        /// The actual property key name is <c>prefix + "extensions"</c>
        /// and multiple values are possible.
        /// </remarks>
        ///
        /// <param name="nameSpace">
        /// The namespace to load configuration values from.
        /// </param>
        /// <param name="prefix">
        /// The prefix of the property name to load configuration values from.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="nameSpace"/> is empty.
        /// </exception>
        /// <exception cref="ConfigurationException">
        /// The property name cannot be found or
        /// it does not hold any values or it contains duplicate values.
        /// </exception>
        public ExtensionMatcher(string nameSpace, string prefix)
        {
            // cannot be null
            if (nameSpace == null)
            {
                throw new ArgumentNullException("nameSpace");
            }
            if (prefix == null)
            {
                throw new ArgumentNullException("prefix");
            }
            // nameSpace cannot be empty
            if (nameSpace.Trim().Length == 0)
            {
                throw new ArgumentException("'nameSpace' cannot be empty", "nameSpace");
            }

            // the property name
            string propName = prefix + ExtensionsPropertyKey;
            try
            {
                // load the multi-value property
                ConfigManager cm = ConfigManager.GetInstance();
                string[] exts = cm.GetValues(nameSpace, propName);

                // required and cannot be empty
                if ((exts == null) || (exts.Length == 0))
                {
                    throw new ConfigurationException(string.Format(
                        "{0} cannot be found or is empty", propName));
                }

                // add list to dictionary
                // (duplicate will throw ArgumentException, encapsulated to ConfigurationException)
                this.extensions = CollectionsUtil.CreateCaseInsensitiveHashtable();
                Helper.AddListToDictionary(exts, this.extensions);
            }
            catch (ConfigurationException)
            {
                // just re-throw
                throw;
            }
            catch (Exception e)
            {
                // encapsulate all others in ConfigurationException
                throw new ConfigurationException(string.Format(
                    "Error initializing from namespace {0}, property {1}", nameSpace, propName), e);
            }
        }

        /// <summary>
        /// Checks whether the filename matches any of the extensions.
        /// </summary>
        ///
        /// <remarks>
        /// The extension is extracted after the last dot in the filename.
        /// If the filename does not contain a dot, the extension is empty.
        /// Matching is carried out in a case-insensitive manner.
        /// </remarks>
        ///
        /// <param name="filename">
        /// The filename whose extension to match.
        /// </param>
        ///
        /// <returns>
        /// <b>true</b> if the file extension match; otherwise, <b>false</b>
        /// </returns>
        protected override bool Match(string filename)
        {
            // get the extension (this may return null, empty, or ext (with .))
            string ext = Path.GetExtension(filename);

            // remove the 1st dot if it exists
            if ((ext != null) && (ext.Length > 0) && (ext[0] == '.'))
            {
                ext = ext.Substring(1);
            }

            // match against the list (O(1) operation)
            return extensions.Contains(ext);
        }
    }
}
