/*
 * FileCategoryMatcher.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using System.IO;

namespace TopCoder.File.Statistics.Matcher
{
    /// <summary>
    /// <see cref="FileCategoryMatcher"/> is a template class
    /// that handles file category (text/binary) oriented matching logic.
    /// </summary>
    ///
    /// <threadsafety static="true" instance="true"/>
    public abstract class FileCategoryMatcher : IFileMatcher
    {
        /// <summary>
        /// Checks whether a file is binary based on its first 1024 bytes.
        /// </summary>
        ///
        /// <remarks>
        /// <para>
        /// If the first 1024 bytes (may be less if the file size is less than 1024 bytes)
        /// of the file content is not within [32, 127]
        /// and is not tab, carriage return, or new line, it's considered a binary file;
        /// otherwise, it's a text file.
        /// An empty file is also considered as a text file.
        /// </para>
        /// <para>
        /// This method is exposed as public static
        /// because it could potentially be used by other custom matchers.
        /// </para>
        /// </remarks>
        ///
        /// <param name="file">
        /// The file to check.
        /// </param>
        ///
        /// <returns>
        /// <b>true</b> if the file is a binary; otherwise, <b>false</b>.
        /// </returns>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="file"/> is a null reference.
        /// </exception>
        /// <exception cref="IOException">
        /// IO operation fails.
        /// </exception>
        public static bool IsBinary(FileInfo file)
        {
            // how many bytes max to test
            const int maxBytes = 1024;

            // cannot be null
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            try
            {
                // assume it's text
                bool result = false;
                using (FileStream stream = file.OpenRead())
                {
                    // read the first maxBytes bytes
                    byte[] buf = new byte[maxBytes];
                    int bytesRead = stream.Read(buf, 0, maxBytes);

                    // check it's [32,127] or tab or CR or LF
                    for (int i = 0; i < bytesRead; i++)
                    {
                        byte b = buf[i];
                        if (!(
                            (b >= 32 && b <= 127) || (b == '\t') || (b == '\n') || (b == '\r')
                            ))
                        {
                            result = true;
                            break;
                        }
                    }
                }

                return result;
            }
            catch (IOException)
            {
                // just re-throw
                throw;

            }
            catch (Exception e)
            {
                // encapsulate all others in IOException
                throw new IOException(string.Format(
                    "Error determining whether file {0} is binary or text", file.FullName), e);
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="FileCategoryMatcher"/> instance.
        /// </summary>
        protected FileCategoryMatcher()
        {
        }

        /// <summary>
        /// Checks whether the file is binary or text using <see cref="IsBinary"/>
        /// and delegates to the template method to decide.
        /// </summary>
        ///
        /// <param name="file">
        /// The file to match.
        /// </param>
        ///
        /// <returns>
        /// <b>true</b> if the file category matches; otherwise, <b>false</b>.
        /// </returns>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="file"/> is a null reference.
        /// </exception>
        /// <exception cref="IOException">
        /// IO operation fails.
        /// </exception>
        public bool Match(FileInfo file)
        {
            // cannot be null
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            try
            {
                // determine file content and call the template method
                return Match(IsBinary(file));
            }
            catch (IOException)
            {
                // just re-throw
                throw;
            }
            catch (Exception e)
            {
                // encapsulate all other errors in IOException
                // we may also get UnauthorizedAccessException
                // if file is a directory, simply wrap it in IOException
                throw new IOException(string.Format(
                    "Error matching for filename {0}", file.FullName), e);
            }
        }

        /// <summary>
        /// The template method which decides whether the file category matches.
        /// </summary>
        ///
        /// <param name="isBinary">
        /// The flag whether the file content is binary.
        /// </param>
        ///
        /// <returns>
        /// <b>true</b> if the file category matches; otherwise, <b>false</b>.
        /// </returns>
        protected abstract bool Match(bool isBinary);
    }
}
