/*
 * RegexMatcher.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using System.Collections;
using System.Text.RegularExpressions;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics.Matcher
{
    /// <summary>
    /// Matches a filename with one or more regular expressions.
    /// </summary>
    ///
    /// <remarks>
    /// It is possible to instantiate this class using reflection
    /// and has its regular expressions list loaded from a configuration file.
    /// The constructor with nameSpace and prefix arguments is used for this purpose.
    /// There should be a multi-value property with the name <c>prefix + "patterns"</c>,
    /// which contains the regular expressions list.
    /// Although the property is required, empty is allowed to match a file with no extension.
    /// </remarks>
    ///
    /// <threadsafety static="true" instance="true"/>
    public class RegexMatcher : FileNameMatcher
    {
        /// <summary>
        /// The configuration key that contains multiple regular expressions to match.
        /// </summary>
        ///
        /// <remarks>
        /// The actual property key name is <c>prefix + "patterns"</c>.
        /// The regexs are compiled when loaded and invalid
        /// pattern will result in a <see cref="ConfigurationException"/>.
        /// </remarks>
        public static readonly string PatternsPropertyKey = "patterns";

        /// <summary>
        /// A list of compiled regular expressions (Regex) to match.
        /// This list is initialized in constructor and is immutable.
        /// </summary>
        private IList patterns;

        /// <summary>
        /// Initializes a new instance of the <see cref="RegexMatcher"/>
        /// with the specified single regular expression.
        /// </summary>
        ///
        /// <param name="pattern">
        /// A single regular expression to match.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="pattern"/> is a null reference.
        /// </exception>
        public RegexMatcher(Regex pattern)
        {
            // cannot be null
            if (pattern == null)
            {
                throw new ArgumentNullException("pattern");
            }

            // save to instance members
            this.patterns = new ArrayList(1);
            this.patterns.Add(pattern);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="RegexMatcher"/>
        /// with the specified list of regular expressions.
        /// </summary>
        ///
        /// <param name="patterns">
        /// The list of regular expressions instances to match.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="patterns"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="patterns"/> is empty or contains non <see cref="Regex"/> instance.
        /// </exception>
        public RegexMatcher(IList patterns)
        {
            // cannot be null
            if (patterns == null)
            {
                throw new ArgumentNullException("patterns");
            }
            // cannot be empty
            if (patterns.Count == 0)
            {
                throw new ArgumentException("'patterns' cannot be an empty list", "patterns");
            }
            // cannot contain non-Regex instance
            foreach (object obj in patterns)
            {
                if (!(obj is Regex))
                {
                    throw new ArgumentException(
                        "'patterns' contains a non Regex instance", "patterns");
                }
            }

            // clone to instance members
            this.patterns = new ArrayList(patterns);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="RegexMatcher"/>
        /// which loads the regular expressions list from the specified nameSpace
        /// and prefix of the configuration.
        /// </summary>
        ///
        /// <remarks>
        /// The actual property key name is <c>prefix + "patterns"</c>
        /// and multiple values are possible.
        /// Each pattern is compiled when loaded.
        /// </remarks>
        ///
        /// <param name="nameSpace">
        /// The namespace to load configuration values from.
        /// </param>
        /// <param name="prefix">
        /// The prefix of the property name to load configuration values from.
        /// </param>
        ///
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="nameSpace"/> is empty.
        /// </exception>
        /// <exception cref="ConfigurationException">
        /// The property name cannot be found or
        /// it does not hold any values or it contains invalid regular expressions.
        /// </exception>
        public RegexMatcher(string nameSpace, string prefix)
        {
            // cannot be null
            if (nameSpace == null)
            {
                throw new ArgumentNullException("nameSpace");
            }
            if (prefix == null)
            {
                throw new ArgumentNullException("prefix");
            }
            // nameSpace cannot be empty
            if (nameSpace.Trim().Length == 0)
            {
                throw new ArgumentException("'nameSpace' cannot be empty", "nameSpace");
            }

            // the property name
            string propName = prefix + PatternsPropertyKey;
            try
            {
                // load the multi-value property
                ConfigManager cm = ConfigManager.GetInstance();
                string[] pats = cm.GetValues(nameSpace, propName);

                // required and cannot be empty
                if ((pats == null) || (pats.Length == 0))
                {
                    throw new ConfigurationException(string.Format(
                        "{0} cannot be found or is empty", propName));
                }

                // compile each pattern and add to list
                this.patterns = new ArrayList(pats.Length);
                foreach (string s in pats)
                {
                    try
                    {
                        patterns.Add(new Regex(s));
                    }
                    catch (Exception e)
                    {
                        throw new ConfigurationException(string.Format(
                            "Error compiling pattern {0}", s), e);
                    }
                }
            }
            catch (ConfigurationException)
            {
                // just re-throw
                throw;
            }
            catch (Exception e)
            {
                // encapsulate all others in ConfigurationException
                throw new ConfigurationException(string.Format(
                    "Error initializing from namespace {0}, property {1}", nameSpace, propName), e);
            }
        }

        /// <summary>
        /// Checks whether the filename matches any of the regular expressions.
        /// </summary>
        ///
        /// <param name="filename">
        /// The filename to match.
        /// </param>
        ///
        /// <returns>
        /// <b>true</b> if the filename match; otherwise, <b>false</b>
        /// </returns>
        protected override bool Match(string filename)
        {
            // test with each regex
            foreach (Regex r in this.patterns)
            {
                if (r.IsMatch(filename))
                {
                    // return immediately upon a match
                    return true;
                }
            }

            // no match
            return false;
        }
    }
}
