/*
 * Metric.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// Represents a measurement of a statistic result.
    /// </summary>
    ///
    /// <remarks>
    /// A metric is identified by its name. Metrics with the same name will be aggregated.
    /// This class overrides <see cref="Equals"/> and <see cref="GetHashCode"/>
    /// so that it can be used as map keys.
    /// </remarks>
    ///
    /// <threadsafety static="true" instance="true"/>
    public class Metric
    {
        /// <summary>
        /// The name of the metric. Acts as an identifier.
        /// Initialized in constructor. Cannot be null nor empty.
        /// </summary>
        private string name;

        /// <summary>
        /// The display name in the rendered result.
        /// If this is used in xml reporting,
        /// the display name should be well formed element name.
        /// Initialized in constructor. Cannot be null nor empty.
        /// </summary>
        private string displayName;

        /// <summary>
        /// A description for the metric.
        /// This does not make a significance in this release.
        /// Initialized in constructor. Cannot be null nor empty.
        /// </summary>
        private string description;

        /// <summary>
        /// Gets the name of the metric.
        /// </summary>
        ///
        /// <remarks>
        /// The name acts an identifier for this metric.
        /// </remarks>
        ///
        /// <value>
        /// The name of the metric.
        /// </value>
        public string Name
        {
            get
            {
                return this.name;
            }
        }

        /// <summary>
        /// Gets the display name of the metric.
        /// </summary>
        ///
        /// <value>
        /// The display name of the metric.
        /// </value>
        public string DisplayName
        {
            get
            {
                return this.displayName;
            }
        }

        /// <summary>
        /// Gets the description of the metric.
        /// </summary>
        ///
        /// <value>
        /// The description of the metric.
        /// </value>
        public string Description
        {
            get
            {
                return this.description;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Metric"/> class.
        /// </summary>
        ///
        /// <param name="name">
        /// The name of the metric.
        /// </param>
        /// <param name="displayName">
        /// The display name of the metric.
        /// </param>
        /// <param name="description">
        /// The description of the metric.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// Any of the argument is empty after trimmed.
        /// </exception>
        public Metric(string name, string displayName, string description)
        {
            // cannot be null
            if (name == null)
            {
                throw new ArgumentNullException("name");
            }
            if (displayName == null)
            {
                throw new ArgumentNullException("displayName");
            }
            if (description == null)
            {
                throw new ArgumentNullException("description");
            }
            // cannot be empty
            if (name.Trim().Length == 0)
            {
                throw new ArgumentException("'name' cannot be empty", "name");
            }
            if (displayName.Trim().Length == 0)
            {
                throw new ArgumentException("'displayName' cannot be empty", "displayName");
            }
            if (description.Trim().Length == 0)
            {
                throw new ArgumentException("'description' cannot be empty", "description");
            }

            // save to instance members
            this.name = name;
            this.displayName = displayName;
            this.description = description;
        }

        /// <summary>
        /// Determines whether two <see cref="Metric"/> objects are equal based on their name.
        /// </summary>
        ///
        /// <param name="obj">
        /// The <see cref="object"/> to compare.
        /// </param>
        ///
        /// <returns>
        /// <b>true</b> if <paramref name="obj"/> is a <see cref="Metric"/>
        /// and has the same name; otherwise, <b>false</b>.
        /// </returns>
        public override bool Equals(object obj)
        {
            // must be a Metric
            if (!(obj is Metric))
            {
                return false;
            }

            // compare the metric name
            return this.name == ((Metric) obj).Name;
        }

        /// <summary>
        /// Returns the hashcode for this instance that is based on its name.
        /// </summary>
        ///
        /// <returns>
        /// A 32-bit signed integer hash code of its name.
        /// </returns>
        public override int GetHashCode()
        {
            // use the metric name
            return this.name.GetHashCode();
        }
    }
}
