/*
 * IStatsReporting.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using System.Collections;
using System.Collections.Specialized;
using System.IO;

namespace TopCoder.File.Statistics.Reporting
{
    /// <summary>
    /// A generic container for mapping from string keys to <see cref="StatsResult"/> instances.
    /// </summary>
    ///
    /// <remarks>
    /// It is used in <see cref="AbstractStatsReporting"/> to hold file, directory,
    /// and alias classified mappings.
    /// The class provides basic manipulation methods for the mapping.
    /// </remarks>
    ///
    /// <threadsafety static="false" instance="false"/>
    public class StatsCollection
    {
        /// <summary>
        /// Mapping from String to StatsResult.
        /// This is used to classify file, dir and alias based stats
        /// in AbstractStatsReporting.
        /// The reference never change.
        /// </summary>
        private IDictionary stats = new Hashtable();

        /// <summary>
        /// Gets the key set of the collection.
        /// </summary>
        ///
        /// <value>
        /// A list of <see cref="string"/> containing the key set of the collection
        /// in alphabetical order (case-insensitive).
        /// </value>
        public IList AllKeys
        {
            get
            {
                // return the sorted case-insensitive string list
                ArrayList result = new ArrayList(stats.Keys);
                result.Sort(CaseInsensitiveComparer.Default);
                return result;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="StatsCollection"/> class.
        /// </summary>
        public StatsCollection()
        {
        }

        /// <summary>
        /// Adds a result to specified key.
        /// </summary>
        ///
        /// <remarks>
        /// If the specified key already exists, <paramref name="stats"/> will be aggregated.
        /// </remarks>
        ///
        /// <param name="key">
        /// The key to associate result with.
        /// </param>
        /// <param name="stats">
        /// The result to add.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="key"/> is empty after trimmed.
        /// </exception>
        public void Add(string key, StatsResult stats)
        {
            // cannot be null nor empty
            if (key == null)
            {
                throw new ArgumentNullException("key");
            }
            if (stats == null)
            {
                throw new ArgumentNullException("stats");
            }
            if (key.Trim().Length == 0)
            {
                throw new ArgumentException("'key' cannot be empty", "key");
            }

            // if exists, aggregate it; otherwise clone it
            if (this.stats.Contains(key))
            {
                ((StatsResult) this.stats[key]).Aggregate(stats);
            }
            else
            {
                this.stats[key] = new StatsResult(stats);
            }
        }

        /// <summary>
        /// Clears the collection.
        /// </summary>
        public void Clear()
        {
            stats.Clear();
        }

        /// <summary>
        /// Gets the <see cref="StatsResult"/> associated with a key.
        /// </summary>
        ///
        /// <param name="key">
        /// The key to retrieve.
        /// </param>
        ///
        /// <value>
        /// The <see cref="StatsResult"/> for the specified key
        /// or <b>null</b> if no result is associated with the key.
        /// </value>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="key"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="key"/> is empty after trimmed.
        /// </exception>
        public StatsResult this[string key]
        {
            get
            {
                // cannot be null nor empty
                if (key == null)
                {
                    throw new ArgumentNullException("key");
                }
                if (key.Trim().Length == 0)
                {
                    throw new ArgumentException("'key' cannot be empty", "key");
                }

                // return the value
                return (StatsResult) this.stats[key];
            }
        }
    }
}
