/*
 * StatsResult.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using System.Collections;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// <see cref="StatsResult"/> is a container for multiple results,
    /// each of which are mapped from a <see cref="Metric"/> to a <see cref="long"/>.
    /// </summary>
    ///
    /// <remarks>
    /// This class wraps some map operations as well as
    /// provides a way to clone and aggregate (add) results.
    /// </remarks>
    ///
    /// <threadsafety static="false" instance="false"/>
    public class StatsResult
    {
        /// <summary>
        /// The results mapping from Metric to the statistical quantity (long).
        /// This reference is never changed.
        /// </summary>
        private IDictionary results = new Hashtable();

        /// <summary>
        /// Gets a list of all the associated metrics.
        /// </summary>
        ///
        /// <value>
        /// A list of <see cref="Metric"/> instances stored in this <see cref="StatsResult"/>.
        /// </value>
        public IList Metrics
        {
            get
            {
                return new ArrayList(results.Keys);
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="StatsResult"/> class.
        /// </summary>
        ///
        /// <remarks>
        /// This contructor will initialize an empty <see cref="StatsResult"/>.
        /// </remarks>
        public StatsResult()
        {
        }

        /// <summary>
        /// Creates a result based on another result.
        /// </summary>
        ///
        /// <remarks>
        /// This acts as a way to clone the instance.
        /// </remarks>
        ///
        /// <param name="result">
        /// The <see cref="StatsResult"/> to clone from.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="result"/> is a null reference.
        /// </exception>
        public StatsResult(StatsResult result)
        {
            // delegate to Aggregate, ANE is also checked there
            Aggregate(result);
        }

        /// <summary>
        /// Aggregates a result into the current result.
        /// </summary>
        ///
        /// <param name="result">
        /// The <see cref="StatsResult"/> to aggregate from.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="result"/> is a null reference.
        /// </exception>
        public void Aggregate(StatsResult result)
        {
            // cannot be null
            if (result == null)
            {
                throw new ArgumentNullException("result");
            }

            // aggregate/add all the result metrics
            IList addMetrics = result.Metrics;
            foreach (Metric addMetric in addMetrics)
            {
                // initialize with 0 if the Metric doesn't exist
                if (!(this.results.Contains(addMetric)))
                {
                    this[addMetric] = 0;
                }
                // aggregate/add the long value
                this[addMetric] += result[addMetric];
            }
        }

        /// <summary>
        /// Gets or sets the value of a certain <see cref="Metric"/>.
        /// </summary>
        ///
        /// <param name="metric">
        /// The <see cref="Metric"/> whose value to get or set.
        /// </param>
        ///
        /// <value>
        /// A long value for the specified <see cref="Metric"/>.
        /// If the specified <see cref="Metric"/> already exists,
        /// its value will be overwritten.
        /// </value>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="metric"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// No value is associated with the specified <paramref name="metric"/>.
        /// </exception>
        public long this[Metric metric]
        {
            get
            {
                // cannot be null
                if (metric == null)
                {
                    throw new ArgumentNullException("metric");
                }
                // throw AE if no value is associated
                if (!(this.results.Contains(metric)))
                {
                    throw new ArgumentException(string.Format(
                        "Cannot find the metric {0}", metric.Name));
                }

                // return the value
                return (long) this.results[metric];
            }
            set
            {
                // cannot be null
                if (metric == null)
                {
                    throw new ArgumentNullException("metric");
                }
                // set the metric value
                this.results[metric] = value;
            }
        }
    }
}
