/*
 * CommandLine.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using System.Collections;
using System.IO;
using TopCoder.Util.ConfigurationManager;
using TopCoder.Util.CommandLine;
using TopCoder.Util.DataValidator;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// The command line entry point of the File Statistics utility.
    /// </summary>
    ///
    /// <remarks>
    /// <para>
    /// It uses <see cref="ExtendedFileStatistics"/> to fulfill the task.
    /// When argument is invalid, help message should be printed.
    /// </para>
    /// <para>
    /// The command line syntax:
    /// <code>
    /// CommandLine.exe [-c:config-file] [-n:namespace] [-b:base-path] [-r] [-o:output-file] path [path2 path3 ...]
    /// where:
    /// * -c:config-file: provides a config file to initialize from
    /// * -n:namespace: provides a namespace to initialize from
    /// *    If neither is provided the default namespace is used.
    /// *    Provided namespace should be preloaded.
    /// * -b:base-path: all path will be relative to it.
    /// * -r: whether to recursively traverse sub-directory.
    /// * -o:output-file: the output filename. If missing, output is printed to console.
    /// * path: the file or directory to process.
    ///   The path is relative to the base path if -b is specified.
    /// </code>
    /// </para>
    /// <para>
    /// Since version 2.0b plugins can register command line options.
    /// This is implemented by the <see cref="IArgumentHandler"/>.
    /// This class handles the logic to collect the proposals, to validate,
    /// and to send the arguments to the handlers.
    /// </para>
    /// </remarks>
    ///
    /// <threadsafety static="true" instance="true"/>
    public class CommandLine
    {
        /// <summary>
        /// The namespace to used if no namespace is provided.
        /// </summary>
        public static readonly string Namespace = "TopCoder.File.Statistics.Main";

        /// <summary>
        /// The command line parser.
        /// Initialized in InitParser().
        /// </summary>
        private static CommandLineParser parser;

        /// <summary>
        /// The ExtendedFileStatistics instance to use.
        /// Initialized in Main();
        /// </summary>
        private static ExtendedFileStatistics statistics;

        /// <summary>
        /// Prevents this class from being instantiated.
        /// </summary>
        private CommandLine()
        {
        }

        /// <summary>
        /// The command line entry.
        /// </summary>
        ///
        /// <param name="args">
        /// The command line arguments. Please refer to the class documentation.
        /// </param>
        public static void Main(string[] args)
        {
            try
            {
                // init CommandLineParser component with internally supported switches
                // and parse the comand line
                InitParser();
                parser.Parse(args);

                // at least 1 argument
                if (parser.Arguments.Length < 1)
                {
                    PrintHelp();
                    return;
                }

                // load configuration file (-c) to the specified namespace (-n)
                // both switches are optional
                LoadConfiguration();

                // determine namespace and instantiate ExtendedStatistics
                string nameSpace = ((parser.InputtedSwitches.Contains("n")) ?
                    GetSwitchValue("n") : Namespace);
                statistics = new ExtendedFileStatistics(nameSpace);

                // set the base path
                string basePath = GetSwitchValue("b");
                if (basePath != null)
                {
                    statistics.Reporting.Reset(new DirectoryInfo(basePath));
                }

                // do we want recurse
                bool recurse = parser.InputtedSwitches.Contains("r");

                // finally, let's process all IArgumentHandler options
                ProcessPluginsOptions(statistics);

                // process all file/dir arguments
                foreach (string arg in parser.Arguments)
                {
                    // make it relative to the base path (if basepath is specified)
                    string file = arg;
                    if (basePath != null)
                    {
                        if (basePath.EndsWith(@"\"))
                        {
                            file = basePath + file;
                        }
                        else
                        {
                            file = basePath + @"\" + file;
                        }
                    }

                    // assume it's a directory
                    DirectoryInfo dir = new DirectoryInfo(file);

                    // use ProcessDirectory or ProcessFile accordingly
                    if ((dir.Attributes & FileAttributes.Directory) != 0)
                    {
                        statistics.ProcessDirectory(dir, recurse);
                    }
                    else
                    {
                        statistics.ProcessFile(new FileInfo(file));
                    }
                }

                // print report
                if (parser.InputtedSwitches.Contains("o"))
                {
                    statistics.PrintReport(new FileInfo(GetSwitchValue("o")));
                }
                else
                {
                    statistics.PrintReport();
                }
            }
            catch (Exception e)
            {
                // print the error, then print the help
                Console.WriteLine(string.Format("Error: {0}{1}{2}",
                    e.Message, Environment.NewLine, e.StackTrace));
                Console.WriteLine();
                PrintHelp();
            }
        }

        /// <summary>
        /// Prints help messages to console.
        /// </summary>
        private static void PrintHelp()
        {
            Console.WriteLine("The command line syntax:");
            Console.WriteLine("CommandLine.exe [-c:config-file] [-n:namespace] [-b:base-path] [-r] "
                + "[-o:output-file] path [path2 path3 ...]");
            Console.WriteLine("where:");
            Console.WriteLine("* -c:config-file: provides a config file to initialize from");
            Console.WriteLine("* -n:namespace: provides a namespace to initialize from");
            Console.WriteLine("*    If neither is provided the default namespace is used.");
            Console.WriteLine("*    Provided namespace should be preloaded.");
            Console.WriteLine("* -b:base-path: all path will be relative to it.");
            Console.WriteLine("* -r: whether to recursively traverse sub-directory.");
            Console.WriteLine("* -o:output-file: the output filename. If missing, output is printed to console.");
            Console.WriteLine("* path: the file or directory to process.");
            Console.WriteLine("  The path is relative to the base path if -b is specified.");

            // now print each handler help message
            if (statistics != null)
            {
                Console.WriteLine();
                Console.WriteLine("Plugin options:");
                IList args = statistics.GetAllArgumentHandlers();
                // don't show duplicate
                IDictionary seen = new Hashtable();
                foreach (IArgumentHandler arg in args)
                {
                    // get all its proposed options
                    IList opts = arg.ProposeOptions();
                    foreach (ArgumentOption opt in opts)
                    {
                        if (seen.Contains(opt.SwitchName))
                        {
                            // skip it
                            continue;
                        }
                        seen.Add(opt.SwitchName, null);
                        Console.WriteLine(string.Format("{0} {1} {2}",
                            opt.SwitchName, opt.HelpMessage, opt.IsRequired ? "(Required)" : ""));
                    }
                }
            }
        }

        /// <summary>
        /// Initializes the command line parser.
        /// </summary>
        private static void InitParser()
        {
            parser = new CommandLineParser();
            // -c
            parser.AddSwitch(new CommandLineSwitch(
                "c", "config-file", "Provides a config file to initialize from"));
            // -n
            parser.AddSwitch(new CommandLineSwitch(
                "n", "namespace", "Provides a namespace to initialize from"));
            // -b
            parser.AddSwitch(new CommandLineSwitch(
                "b", "base-path", "All path will be relative to it"));
            // -r
            parser.AddSwitch(new CommandLineSwitch(
                "r", "recurse", "Whether to recursively traverse sub-directory"));
            // -o
            parser.AddSwitch(new CommandLineSwitch(
                "o", "output-file", "The output filename. If missing, output is printed to console."));
        }

        /// <summary>
        /// Loads configuration as specified by the command line argument.
        /// </summary>
        private static void LoadConfiguration()
        {
            ConfigManager cm = ConfigManager.GetInstance();
            if (parser.InputtedSwitches.Contains("c"))
            {
                // is namespace also specified?
                if (parser.InputtedSwitches.Contains("n"))
                {
                    // load config into the specified namespace
                    cm.LoadFile(GetSwitchValue("c"), GetSwitchValue("n"));
                }
                else
                {
                    // just load it
                    cm.LoadFile(GetSwitchValue("c"));
                }
            }
        }

        /// <summary>
        /// Get switch value as a string
        /// </summary>
        ///
        /// <param name="sw">
        /// The switch.
        /// </param>
        ///
        /// <returns>
        /// The switch value as a string, or null if the switch isn't supplied.
        /// </returns>
        private static string GetSwitchValue(string sw)
        {
            if (!(parser.InputtedSwitches.Contains(sw)))
            {
                // switch isn't supplied
                return null;
            }

            // get the switch value
            return (string) ((CommandLineSwitch) parser.InputtedSwitches[sw]).Result[0];
        }

        /// <summary>
        /// Processes all handler's options.
        /// </summary>
        ///
        /// <param name="statistics">
        /// The <see cref="FileStatistics"/> instance to process.
        /// </param>
        private static void ProcessPluginsOptions(FileStatistics statistics)
        {
            IList args = statistics.GetAllArgumentHandlers();
            foreach (IArgumentHandler arg in args)
            {
                // get all its proposed options, and its command-line value
                IList opts = arg.ProposeOptions();
                IDictionary handlerArguments = new Hashtable();
                foreach (ArgumentOption opt in opts)
                {
                    string switchVal = GetSwitchValue(opt.SwitchName);

                    // required but not exists -> throw error
                    if (opt.IsRequired && ((switchVal == null) || (switchVal.Trim().Length == 0)))
                    {
                        throw new Exception(string.Format(
                            "Missing required argument value for {0}: {1}", opt.SwitchName, opt.HelpMessage));
                    }
                    // validate argument
                    if (opt.Validator != null)
                    {
                        if (!opt.Validator.IsValid(switchVal))
                        {
                            throw new Exception(string.Format(
                                "Error validating argument value for {0}: {1}", opt.SwitchName, opt.HelpMessage));
                        }
                    }
                    // add the handler value
                    handlerArguments.Add(opt.SwitchName, switchVal);
                }

                // configure the handler
                arg.HandleArguments(handlerArguments);
            }
        }
    }
}
