/*
 * CStyleLineCounter.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using System.Collections;
using System.IO;
using TopCoder.File.Statistics.Matcher;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics.Processor.LineCounter
{
    /// <summary>
    /// Counts lines for C style source files.
    /// </summary>
    ///
    /// <remarks>
    /// <para>
    /// This implementation might be able to handle C, C++, C#, or Java source code.
    /// However, it only takes opening braces and semicolons into account
    /// so it might not be very accurate in most cases,
    /// for example the statement <c>for (int i = 0; i &lt;= 5; i++) { test(); }</c>
    /// will be counted as 4 lines.
    /// Comments and string literals can be handled, so that opening braces and semicolons
    /// inside them are ignored as expected.
    /// It supports a single metric of "Line Count".
    /// It delegates configuration to base class without any addition.
    /// </para>
    /// <para>
    /// The code parser is implemented using a simple state machine.
    /// </para>
    /// <para>
    /// Since version 2.0b this class adds an option to support the at style literal (literal like @"..."
    /// that does not enable escape characters).
    /// This option can come from either configuration file or command line.
    /// </para>
    /// </remarks>
    ///
    /// <threadsafety static="true" instance="true"/>
    public class CStyleLineCounter : FileProcessorBase, IArgumentHandler
    {
        /// <summary>
        /// Configuration property key that holds a boolean flag whether to support
        /// at style literal (@"...").
        /// </summary>
        ///
        /// <remarks>
        /// This is optional and is evaluated to <b>true</b>
        /// if the value is "true"/"yes"/"on" case sensitively.
        /// Default value is <b>false</b>.
        /// </remarks>
        public static readonly string SupportAtLiteralPropertyKey = "support_at_literal";

        /// <summary>
        /// The name of the metric used for this processor.
        /// </summary>
        public static readonly string Name = "Line Count";

        /// <summary>
        /// The display name of the metric used for this processor.
        /// </summary>
        public static readonly string DisplayName = "line";

        /// <summary>
        /// The description of the metric used for this processor.
        /// </summary>
        private static readonly string description = "Number of statements in a source code file.";

        /// <summary>
        /// The supported metric list for this processor.
        /// </summary>
        private static readonly Metric[] supportedMetric =
            { new Metric(Name, DisplayName, description) };

        /// <summary>
        /// Flag whether to support at style literal.
        /// It's set in constructor or HandleArguments(). Default to false.
        /// </summary>
        private bool supportAtLiteral = false;

        /// <summary>
        /// The help message for the option key.
        /// </summary>
        private static readonly string helpOption =
            "Whether to support C#-style at-quote literal (\"yes\"/\"true\"/\"on\" is considered true)";

        /// <summary>
        /// Gets a list containing a single <see cref="Metric"/> composed of
        /// the <see cref="Name"/>, <see cref="DisplayName"/>, and a description.
        /// </summary>
        ///
        /// <value>
        /// A list containing a single <see cref="Metric"/> composed of
        /// the <see cref="Name"/>, <see cref="DisplayName"/>, and a description.
        /// </value>
        public override IList SupportedMetrics
        {
            get
            {
                // return the supported metric list
                return supportedMetric;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="CStyleLineCounter"/> instance
        /// with the specified alias name and <see cref="IFileMatcher"/>.
        /// </summary>
        ///
        /// <param name="alias">
        /// The alias name.
        /// </param>
        /// <param name="matcher">
        /// The <see cref="IFileMatcher"/> to use.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="alias"/> is empty after trimmed.
        /// </exception>
        public CStyleLineCounter(string alias, IFileMatcher matcher)
            : base(alias, matcher)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="CStyleLineCounter"/> instance
        /// with the specified alias name, <see cref="IFileMatcher"/>, and support at literal flag.
        /// </summary>
        ///
        /// <param name="alias">
        /// The alias name.
        /// </param>
        /// <param name="matcher">
        /// The <see cref="IFileMatcher"/> to use.
        /// </param>
        /// <param name="supportAtLiteral">
        /// A boolean flag to indicate whether at-style string is supported.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="alias"/> is empty after trimmed.
        /// </exception>
        public CStyleLineCounter(string alias, IFileMatcher matcher, bool supportAtLiteral)
            : base(alias, matcher)
        {
            this.supportAtLiteral = supportAtLiteral;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="CStyleLineCounter"/>
        /// which loads configuration from the specified nameSpace
        /// and prefix of the configuration.
        /// </summary>
        ///
        /// <remarks>
        /// The alias name will be loaded from the property <c>prefix + "alias"</c>,
        /// while the file matcher class will be loaded from the property
        /// <c>prefix + "matcher.classname"</c> passing the nameSpace and
        /// <c>prefix + "matcher."</c> to the file matcher constructor.
        /// If the property <c>prefix + "matcher.classname"</c> is missing,
        /// <see cref="AnyFileMatcher"/> will be used.
        /// Since version 2.0b, there is a support for C# style at-literal string.
        /// This is specified in the configuration file in the property
        /// <c>prefix + "support_at_literal"</c>.
        /// The value "true"/"yes"/"on" case insensitively is considered true.
        /// It is optional and default to false.
        /// </remarks>
        ///
        /// <param name="nameSpace">
        /// The namespace to load configuration values from.
        /// </param>
        /// <param name="prefix">
        /// The prefix of the property name to load configuration values from.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// Any of the argument is empty after trimmed.
        /// </exception>
        /// <exception cref="ConfigurationException">
        /// A configuration error occurs, such as missing alias name,
        /// empty alias name, error instantiating file matcher.
        /// </exception>
        public CStyleLineCounter(string nameSpace, string prefix)
            : base(nameSpace, prefix)
        {
            try
            {
                ConfigManager cm = ConfigManager.GetInstance();

                // get support_at_literal
                string propName = prefix + SupportAtLiteralPropertyKey;
                string supportAt = cm.GetValue(nameSpace, propName);
                if (supportAt != null)
                {
                    // true if "yes"/"on"/"true" case-insensitively
                    supportAtLiteral = Helper.BoolValue(supportAt);
                }
            }
            catch (Exception e)
            {
                // encapsulate all in ConfigurationException
                throw new ConfigurationException(string.Format(
                    "Error initializing from namespace {0}, property prefix {1}",
                    nameSpace, prefix), e);
            }
        }

        /// <summary>
        /// Count the lines in C style source code file.
        /// </summary>
        ///
        /// <remarks>
        /// <para>
        /// This will assume that the specified file is a text file.
        /// It would open the file as a text stream and count the code lines until EOF is met.
        /// Code lines is only considered when a semicolon or opening brace is found.
        /// Semicolon or opening brace inside a string literal or comment sections are ignored.
        /// </para>
        /// <para>
        /// Since version 2.0b this method should take the supportAtLiteral flag into consideration.
        /// </para>
        /// </remarks>
        ///
        /// <param name="file">
        /// The file to process.
        /// </param>
        ///
        /// <returns>
        /// A <see cref="StatsResult"/> representing the number of code lines in the file.
        /// </returns>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="file"/> is a null reference.
        /// </exception>
        /// <exception cref="IOException">
        /// Any IO operation fails.
        /// </exception>
        /// <exception cref="FileProcessingException">
        /// The processor fails.
        /// </exception>
        public override StatsResult Process(FileInfo file)
        {
            // cannot be null
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            try
            {
                // open as text and count the code line with simple state machine
                IStateMachine machine = new SimpleStateMachine();
                if (supportAtLiteral)
                {
                    machine = new SimpleStateMachineWithAtLiteral();
                }

                using (StreamReader stream = file.OpenText())
                {
                    string line = stream.ReadLine();
                    while (line != null)
                    {
                        machine.Process(line);
                        line = stream.ReadLine();
                    }
                }

                // return the StatsResult
                StatsResult result = new StatsResult();
                result[(Metric) SupportedMetrics[0]] = machine.Lines;
                return result;
            }
            catch (IOException)
            {
                // just re-throw
                throw;
            }
            catch (Exception e)
            {
                // encapsulate all others in IOException
                throw new IOException(string.Format(
                    "Error processing file {0}", file.FullName), e);
            }

        }

        /// <summary>
        /// Proposes a single <see cref="ArgumentOption"/> with "support_at_literal"
        /// as switch name and non-required option.
        /// </summary>
        ///
        /// <returns>
        /// A <see cref="IList"/> containing a single <see cref="ArgumentOption"/>
        /// with "support_at_literal" as switch name and non-required option.
        /// </returns>
        public IList ProposeOptions()
        {
            IList options = new ArrayList();
            options.Add(new ArgumentOption(SupportAtLiteralPropertyKey, helpOption, false));
            return options;
        }

        /// <summary>
        /// Receives command line argument from the command line interface.
        /// </summary>
        ///
        /// <remarks>
        /// It does not care about the actual value of the switch (which is generally not provided).
        /// It only cares about whether the key option exists (on) or not (off).
        /// </remarks>
        ///
        /// <param name="arguments">
        /// The arguments from command line.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="arguments"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// If the "ignore_blank_line" value is not a String.
        /// </exception>
        public void HandleArguments(IDictionary arguments)
        {
            if (arguments == null)
            {
                throw new ArgumentNullException("arguments");
            }
            object support = arguments[SupportAtLiteralPropertyKey];
            if ((support != null) && (!(support is string)))
            {
                throw new ArgumentException("'support_at_literal' value is not of type string", "arguments");
            }

            // turn on option if the key exists
            if (support != null)
            {
                supportAtLiteral = true;
            }
        }
    }
}
