/*
 * XmlStatsReporting.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Authors: WishingBone, TCSDEVELOPER
 */

using System;
using System.Collections;
using System.IO;
using System.Xml;
using System.Xml.Xsl;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics.Reporting
{
    /// <summary>
    /// A reporting implementation that renders XML reports.
    /// </summary>
    ///
    /// <remarks>
    /// <para>
    /// It is capable of using a stylesheet to transform the XML into
    /// user-oriented format (HTML, CSV, etc).
    /// </para>
    /// <para>
    /// It is possible to initialize this class using a configuration value
    /// using the constructor that takes the nameSpace and prefix arguments.
    /// The constructor with nameSpace and prefix arguments is used for this purpose.
    /// It supports a single value property with the name <c>prefix + "stylesheet"</c>,
    /// which contains a filename of the stylesheet. This property is optional.
    /// If it's missing the raw XML is generated.
    /// </para>
    /// <para>
    /// Here's an example of the raw XML that is generated:
    /// <code>
    /// &lt;report&gt;
    ///     &lt;files&gt;
    ///         &lt;file name="a\p.txt"&gt;
    ///             &lt;line&gt;6&lt;/line&gt;
    ///         &lt;/file&gt;
    ///         &lt;file name="a\q.java"&gt;
    ///             &lt;line&gt;18&lt;/line&gt;
    ///         &lt;/file&gt;
    ///         &lt;file name="b\test.sql"&gt;
    ///             &lt;line&gt;22&lt;/line&gt;
    ///         &lt;/file&gt;
    ///     &lt;/files&gt;
    ///     &lt;directories&gt;
    ///         &lt;directory name="a"&gt;
    ///             &lt;line&gt;24&lt;/line&gt;
    ///         &lt;/directory&gt;
    ///         &lt;directory name="b"&gt;
    ///             &lt;line&gt;22&lt;/line&gt;
    ///         &lt;/directory&gt;
    ///     &lt;/directories&gt;
    ///     &lt;aliases&gt;
    ///         &lt;alias name="Text File"&gt;
    ///             &lt;line&gt;28&lt;/line&gt;
    ///         &lt;/alias&gt;
    ///         &lt;alias name="Source File"&gt;
    ///             &lt;line&gt;18&lt;/line&gt;
    ///         &lt;/alias&gt;
    ///     &lt;/aliases&gt;
    ///     &lt;overall&gt;
    ///         &lt;line&gt;46&lt;/line&gt;
    ///     &lt;/overall&gt;
    /// &lt;/report&gt;
    /// </code>
    /// The statistic's element tag will be created using its DisplayName.
    /// The file and directory paths will be relative paths to the directory
    /// specified when calling <see cref="AbstractStatsReporting.Reset"/>.
    /// However, if <see cref="AbstractStatsReporting.Reset"/> is called with a <b>null</b> value,
    /// all the paths and directory results will be absolute paths.
    /// Also note that this base directory will not get included in the directories
    /// section of the report, as it's the same as the overall section.
    /// </para>
    /// <para>
    /// Since version 2.0b this reporting supports command line input
    /// for the style sheet.
    /// </para>
    /// </remarks>
    ///
    /// <threadsafety static="false" instance="false"/>
    public class XmlStatsReporting : AbstractStatsReporting, IArgumentHandler
    {
        /// <summary>
        /// The property which specifies a stylesheet to apply on the xml.
        /// </summary>
        ///
        /// <remarks>
        /// If this property is not specified, the raw xml output is generated.
        /// </remarks>
        public static readonly string StylesheetPropertyKey = "stylesheet";

        /// <summary>
        /// The stylesheet to apply on the raw xml generated.
        /// This is initialized in constructor and never changed.
        /// The value can be null indicating no stylesheet is used.
        /// </summary>
        private FileInfo stylesheet = null;

        /// <summary>
        /// The help message for the option key.
        /// </summary>
        private static readonly string helpOption = "The stylesheet file name";

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlStatsReporting"/> class.
        /// </summary>
        public XmlStatsReporting()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlStatsReporting"/> class
        /// using the specified stylesheet.
        /// </summary>
        ///
        /// <param name="stylesheet">
        /// The stylesheet to apply.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="stylesheet"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="stylesheet"/> is not a valid file.
        /// </exception>
        public XmlStatsReporting(FileInfo stylesheet)
        {
            // cannot be null
            if (stylesheet == null)
            {
                throw new ArgumentNullException("stylesheet");
            }
            // check valid (exists)
            if (!(stylesheet.Exists))
            {
                throw new ArgumentException(string.Format(
                    "{0} doesn't exist", stylesheet.FullName), "stylesheet");
            }

            // save to instance vars
            this.stylesheet = stylesheet;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlStatsReporting"/>
        /// which loads the stylesheet file from the specified nameSpace
        /// and prefix of the configuration.
        /// </summary>
        ///
        /// <remarks>
        /// The actual property key name is <c>prefix + "stylesheet"</c>.
        /// This property is optional, but should be an existing filename if exists.
        /// </remarks>
        ///
        /// <param name="nameSpace">
        /// The namespace to load configuration values from.
        /// </param>
        /// <param name="prefix">
        /// The prefix of the property name to load configuration values from.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// Any of the argument is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="nameSpace"/> is empty.
        /// </exception>
        /// <exception cref="ConfigurationException">
        /// A configuration error occurs, for example the stylesheet file doesn't exist.
        /// </exception>
        public XmlStatsReporting(string nameSpace, string prefix)
            : base(nameSpace, prefix)
        {
            // ANE/AE is checked by base class

            // the property name for stylesheet
            string propName = prefix + StylesheetPropertyKey;
            try
            {
                // load the single value property
                ConfigManager cm = ConfigManager.GetInstance();
                string stylesheet = cm.GetValue(nameSpace, propName);

                // optional
                if (stylesheet == null)
                {
                    return;
                }

                // file must exist
                FileInfo styleFile = new FileInfo(stylesheet);
                if (!(styleFile.Exists))
                {
                    throw new ConfigurationException(string.Format(
                        "{0} doesn't exist", stylesheet));
                }

                // save to instance vars
                this.stylesheet = styleFile;
            }
            catch (ConfigurationException)
            {
                // just re-throw
                throw;
            }
            catch (Exception e)
            {
                // encapsulate all others in ConfigurationException
                throw new ConfigurationException(string.Format(
                    "Error initializing from namespace {0}, property {1}", nameSpace, propName), e);
            }
        }

        /// <summary>
        /// Generates an XML report for all the aggregated results
        /// if no stylesheet is specified or generates the result
        /// from the XSLT file.
        /// </summary>
        ///
        /// <returns>
        /// The generated report.
        /// </returns>
        ///
        /// <exception cref="StatsReportingException">
        /// An error occurs.
        /// </exception>
        public override string GenerateReport()
        {
            try
            {
                // generate the raw xml
                XmlDocument result = GenerateXmlReport();

                //Create an XmlTextWriter which outputs to a StringWriter
                StringWriter stringWriter = new StringWriter();
                XmlTextWriter xmlWriter = new XmlTextWriter(stringWriter);
                // use pretty printing
                xmlWriter.Formatting = Formatting.Indented;
                xmlWriter.Indentation = 4;

                if (stylesheet == null)
                {
                    // no stylesheet, just return the pretty-formatted raw xml
                    result.WriteTo(xmlWriter);
                    return stringWriter.ToString();
                }

                // load the stylesheet
                XslTransform xslt = new XslTransform();
                xslt.Load(stylesheet.FullName);

                //Transform the file
                xslt.Transform(result, null, xmlWriter, null);

                // return the string result
                return stringWriter.ToString();
            }
            catch (Exception e)
            {
                throw new StatsReportingException("Error generating XML report", e);
            }
        }

        /// <summary>
        /// Generates the raw xml report.
        /// </summary>
        ///
        /// <returns>
        /// A string representing the raw xml document.
        /// </returns>
        private XmlDocument GenerateXmlReport()
        {
            // we use XmlDocument instead of XmlWriter
            // so that it's easier to transform using Xslt later
            XmlDocument doc = new XmlDocument();

            // load the root document element
            doc.LoadXml(@"<report></report>");

            // create the xml declaration: <?xml ... ?>
            XmlDeclaration xmlDecl = doc.CreateXmlDeclaration("1.0", "UTF-8", null);
            doc.InsertBefore(xmlDecl, doc.DocumentElement);

            // create files, directories, aliases section
            AddXmlSection(doc, "files", "file", FileStats);
            AddXmlSection(doc, "directories", "directory", DirectoryStats);
            AddXmlSection(doc, "aliases", "alias", AliasStats);

            // add the overall section and its metrics
            XmlElement overallTag = doc.CreateElement("overall");
            doc.DocumentElement.AppendChild(overallTag);
            AddXmlMetric(doc, overallTag, OverallStats);

            // ok, we're done
            return doc;
        }

        /// <summary>
        /// Creates the xml for a statistic section (file, directory, or alias).
        /// </summary>
        ///
        /// <param name="doc">
        /// The XML document to add.
        /// </param>
        /// <param name="root">
        /// The name of the root tag for the section.
        /// </param>
        /// <param name="element">
        /// The element name of each tag (keys).
        /// </param>
        /// <param name="result">
        /// The StatsCollection containing the StatsResult.
        /// </param>
        private void AddXmlSection(XmlDocument doc, string root, string element, StatsCollection result)
        {
            // create root tag, attach to doc
            XmlNode rootTag = doc.CreateElement(root);
            doc.DocumentElement.AppendChild(rootTag);

            // iterate StatsCollection and create the element
            IList keys = result.AllKeys;
            foreach (string key in keys)
            {
                // create the element tag with name = "key" attribute
                XmlElement elementTag = doc.CreateElement(element);
                elementTag.SetAttribute("name", key);
                // attach to the root tag
                rootTag.AppendChild(elementTag);

                // create metric and attach
                AddXmlMetric(doc, elementTag, result[key]);
            }
        }

        /// <summary>
        /// Creates the xml for each metric and add to the specified xml element.
        /// </summary>
        ///
        /// <param name="doc">
        /// The XML document to add.
        /// </param>
        /// <param name="elementRoot">
        /// The xml element to attach each metric to.
        /// </param>
        /// <param name="result">
        /// The StatsResult containing Metrics to write.
        /// </param>
        private void AddXmlMetric(XmlDocument doc, XmlElement elementRoot, StatsResult result)
        {
            // iterate each metric
            IList metrics = result.Metrics;
            foreach (Metric metric in metrics)
            {
                // create the xml element with the Metric's display name
                // and the value as its inner text
                XmlElement metricTag = doc.CreateElement(metric.DisplayName);
                metricTag.InnerText = result[metric].ToString();

                // attach to element tag
                elementRoot.AppendChild(metricTag);
            }
        }

        /// <summary>
        /// Proposes a single <see cref="ArgumentOption"/> with "stylesheet"
        /// as switch name and non-required option.
        /// </summary>
        ///
        /// <returns>
        /// A <see cref="IList"/> containing a single <see cref="ArgumentOption"/>
        /// with "stylesheet" as switch name and non-required option.
        /// </returns>
        public IList ProposeOptions()
        {
            IList options = new ArrayList();
            options.Add(new ArgumentOption(StylesheetPropertyKey, helpOption, false));
            return options;
        }

        /// <summary>
        /// Receives command line argument from the command line interface.
        /// </summary>
        ///
        /// <remarks>
        /// If no 'stylesheet' key is specified, nothing is done.
        /// However if one is specified, the file must exist.
        /// </remarks>
        ///
        /// <param name="arguments">
        /// The arguments from command line.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="arguments"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// If the "stylesheet" value is not a String.
        /// </exception>
        /// <exception cref="InvalidArgumentException">
        /// If the specified stylesheet file does not exist.
        /// </exception>
        public void HandleArguments(IDictionary arguments)
        {
            if (arguments == null)
            {
                throw new ArgumentNullException("arguments");
            }
            object file = arguments[StylesheetPropertyKey];
            if (file == null)
            {
                return;
            }
            if (!(file is string))
            {
                throw new ArgumentException("'stylesheet' value is not of type string", "arguments");
            }

            stylesheet = new FileInfo((string) file);

            // check file exists
            if (!stylesheet.Exists)
            {
                throw new InvalidArgumentException(string.Format(
                    "Cannot find the specified stylesheet file {0}", file));
            }
        }
    }
}
