/*
 * Demo.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.Collections;
using System.IO;
using System.Text.RegularExpressions;
using NUnit.Framework;
using TopCoder.File.Statistics.Matcher;
using TopCoder.File.Statistics.Processor.LineCounter;
using TopCoder.File.Statistics.Reporting;
using TopCoder.File.Statistics.Tests;

namespace TopCoder.File.Statistics.Tests
{
    /// <summary>
    /// A demonstration of the component functionality.
    /// </summary>
    [TestFixture]
    public class Demo : TestHelper
    {
        /// <summary>
        /// The <see cref="FileStatistics"/> instance to demo.
        /// </summary>
        private FileStatistics statistics;

        /// <summary>
        /// The <see cref="ExtendedFileStatistics"/> instance to demo.
        /// </summary>
        private ExtendedFileStatistics extended;

        /// <summary>
        /// Demonstrates <see cref="FileStatistics"/> configuration.
        /// </summary>
        public void ConfigureFileStatistics()
        {
            // create a file statistics instance
            statistics = new FileStatistics();

            Assert.AreEqual(0, statistics.AllFileProcessors.Count, "No processor");
            Assert.IsFalse(statistics.IsMatchAll, "MatchAll default to false");
            Assert.IsTrue(statistics.Reporting is XmlStatsReporting, "Default reporting");

            // create file matchers
            IFileMatcher matcher1 = new AnyFileMatcher();
            IFileMatcher matcher2 = new ExtensionMatcher("txt");
            IFileMatcher matcher3 = new RegexMatcher(new Regex("pattern"));
            IFileMatcher matcher4 = new TextFileMatcher();
            IFileMatcher matcher5 = new BinaryFileMatcher();
            IFileMatcher matcher6 = new TypeNameMatcher("Java");
            IFileMatcher matcher7 = new MimeMatcher("text/plain");

            Assert.IsTrue(matcher1.Match(new FileInfo(SampleText)), "Always match");
            Assert.IsTrue(matcher2.Match(new FileInfo(SampleText)), ".txt match");
            Assert.IsFalse(matcher3.Match(new FileInfo(SampleText)), "Only pattern");
            Assert.IsTrue(matcher4.Match(new FileInfo(SampleText)), "A text file");
            Assert.IsFalse(matcher5.Match(new FileInfo(SampleText)), "A text file");
            Assert.IsFalse(matcher6.Match(new FileInfo(SampleText)), "Not java");
            Assert.IsFalse(matcher7.Match(new FileInfo(SampleText)),
                "MagicNumber not configured");

            // create file processors
            IFileProcessor processor1 = new TextLineCounter("Text File", matcher1);
            IFileProcessor processor2 = new CStyleLineCounter("Source File", matcher6);

            Assert.AreEqual("Text File", processor1.Alias, "processor1 alias name");
            Assert.AreSame(matcher1, processor1.FileMatcher, "processor1's file matcher");
            Assert.AreEqual("Source File", processor2.Alias, "processor2 alias name");
            Assert.AreSame(matcher6, processor2.FileMatcher, "processor2's file matcher");

            // create stats reporting implementation
            IStatsReporting reporting1 = new BasicStatsReporting();
            IStatsReporting reporting2 = new XmlStatsReporting(
                new FileInfo(@"..\..\test_files\FileStatistics_Stylesheet.xsl"));

            // manipulate file processors
            statistics.AddFileProcessor(processor1);
            statistics.AddFileProcessor(processor2);
            Assert.AreEqual(2, statistics.AllFileProcessors.Count, "2 processor added");

            IFileProcessor removed = statistics.RemoveFileProcessor("Source File");
            Assert.AreEqual(1, statistics.AllFileProcessors.Count, "1 processor left");
            Assert.AreSame(processor2, removed, "The removed processor is returned");

            IFileProcessor query = statistics["Text File"];
            Assert.AreSame(processor1, query, "The same text file processor");

            IList processors = statistics.AllFileProcessors;
            Assert.AreEqual(1, processors.Count, "1 processor left");

            statistics.ClearAllFileProcessors();
            Assert.AreEqual(0, statistics.AllFileProcessors.Count, "0 processor left");

            // manipulate reporting
            IStatsReporting original = statistics.Reporting;
            Assert.IsTrue(original is XmlStatsReporting, "The default reporting");

            statistics.Reporting = reporting2;
            Assert.AreSame(reporting2, statistics.Reporting, "The new reporting");

            // manipulate match all flag
            if (!statistics.IsMatchAll)
            {
                statistics.IsMatchAll = true;
            }
            Assert.IsTrue(statistics.IsMatchAll, "Match all is true now");
        }

        /// <summary>
        /// Demonstrates processing a file.
        /// </summary>
        public void ProcessFile()
        {
            // process single file
            statistics.ProcessFile(new FileInfo(SampleText));
            statistics.ProcessFile(new FileInfo(@"..\..\test_files\test\aaa\sample.txt"));

            // process directory
            extended = new ExtendedFileStatistics(ValidNamespace);
            extended.ProcessDirectory(new DirectoryInfo(@"..\..\test_files\test"));
            extended.ProcessDirectory(new DirectoryInfo(@"..\..\test_files\test"), true);
        }

        /// <summary>
        /// Demonstrates generating report.
        /// </summary>
        public void GenerateReport()
        {
            try
            {
                // obtain report
                string report = statistics.Reporting.GenerateReport();

                // output report
                extended.PrintReport();
                extended.PrintReport(new FileInfo(@"..\..\test_files\TempOutput.txt"));

                // reset reporting
                statistics.Reporting.Reset(new DirectoryInfo(@"..\..\test_files"));
            }
            finally
            {
                System.IO.File.Delete(@"..\..\test_files\TempOutput.txt");
            }
        }

        /// <summary>
        /// Runs the demonstration.
        /// </summary>
        [Test]
        public void RunDemo()
        {
            // load configuration files
            base.LoadConfigFile(MagicNumberConfig, ValidConfig);

            ConfigureFileStatistics();
            ProcessFile();
        }
    }
}
