/*
 * TestAnyFileMatcher.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.IO;
using NUnit.Framework;
using TopCoder.File.Statistics.Tests;

namespace TopCoder.File.Statistics.Matcher.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="AnyFileMatcher"/> class.
    /// </summary>
    [TestFixture]
    public class TestAnyFileMatcher : TestHelper
    {
        /// <summary>
        /// The <see cref="AnyFileMatcher"/> instance to test.
        /// Initialized in SetUp().
        /// </summary>
        private IFileMatcher matcher;

        /// <summary>
        /// Initializes the <see cref="AnyFileMatcher"/> instance to test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            matcher = new AnyFileMatcher();
        }

        /// <summary>
        /// Tests constructor() doesn't throw any exception.
        /// </summary>
        [Test]
        public void TestConstructor1()
        {
            new AnyFileMatcher();
        }

        /// <summary>
        /// Tests constructor(nameSpace, prefix) doesn't throw any exception
        /// given valid inputs.
        /// </summary>
        [Test]
        public void TestConstructor2()
        {
            new AnyFileMatcher(ValidNamespace, "processor1.");
        }

        /// <summary>
        /// Tests constructor(nameSpace, prefix) throws ArgumentNullException
        /// given null namespace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor2NullNamespace()
        {
            new AnyFileMatcher(null, "processor1.");
        }

        /// <summary>
        /// Tests constructor(nameSpace, prefix) throws ArgumentNullException
        /// given null prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor2NullPrefix()
        {
            new AnyFileMatcher(ValidNamespace, null);
        }

        /// <summary>
        /// Tests constructor(nameSpace, prefix) throws ArgumentException
        /// given empty namespace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor2EmptyNamespace()
        {
            new AnyFileMatcher(" ", "processor1.");
        }

        /// <summary>
        /// Tests constructor(nameSpace, prefix) doesn't throw any exception
        /// given empty prefix.
        /// </summary>
        [Test]
        public void TestConstructor2EmptyPrefix()
        {
            new AnyFileMatcher(ValidNamespace, "");
        }

        /// <summary>
        /// Tests constructor(nameSpace, prefix) doesn't throw any exception
        /// given empty-space prefix.
        /// </summary>
        [Test]
        public void TestConstructor2EmptySpacePrefix()
        {
            new AnyFileMatcher(ValidNamespace, " ");
        }

        /// <summary>
        /// Tests Match() returns true given existing file.
        /// </summary>
        [Test]
        public void TestMatch()
        {
            Assert.IsTrue(matcher.Match(new FileInfo(SampleText)),
                "Match() should always returns true");
        }

        /// <summary>
        /// Tests Match() always returns true given non-existing file.
        /// </summary>
        [Test]
        public void TestMatchNotExist()
        {
            Assert.IsTrue(matcher.Match(new FileInfo(@"..\..\test_files\notexists.txt")),
                "Match() should always returns true");
        }

        /// <summary>
        /// Tests Match() returns true although the FileInfo represents
        /// an existing directory.
        /// </summary>
        [Test]
        public void TestMatchDirectory()
        {
            Assert.IsTrue(matcher.Match(new FileInfo(@"..\..\test_files")),
                "Match() should always returns true");
        }

        /// <summary>
        /// Tests Match() returns true although the FileInfo represents
        /// a non existing directory.
        /// </summary>
        [Test]
        public void TestMatchDirectoryNotExist()
        {
            Assert.IsTrue(matcher.Match(new FileInfo(@"..\..\notexists")),
                "Match() should always returns true");
        }

        /// <summary>
        /// Tests Match() throws ArgumentNullException
        /// given null FileInfo.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestMatchNull()
        {
            matcher.Match(null);
        }
    }
}
