/*
 * TestExtensionMatcher.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.Collections;
using System.IO;
using NUnit.Framework;
using TopCoder.File.Statistics.Tests;

namespace TopCoder.File.Statistics.Matcher.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="ExtensionMatcher"/> class.
    /// </summary>
    [TestFixture]
    public class TestExtensionMatcher : TestHelper
    {
        /// <summary>
        /// The <see cref="ExtensionMatcher"/> instance to test.
        /// Initialized in each unit test case according to its need.
        /// </summary>
        private IFileMatcher matcher;

        /// <summary>
        /// A list of extensions to match.
        /// Initialized in SetUp() and used to test constructor(IList).
        /// </summary>
        private IList exts;

        /// <summary>
        /// Initializes the <see cref="ExtensionMatcher"/> instance to test
        /// using the test_files/Matcher_Extension.xml.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            // load test_files/Matcher_Extension.xml
            // used for constructor with nameSpace and prefix argument
            base.LoadConfigFile(@"..\..\test_files\Matcher_Extension.xml");

            // initialize extensions list (with .txt and .c)
            exts = new ArrayList();
            exts.Add("txT");    // we use .txT to test case-insensitivity
            exts.Add("c");
        }

        /// <summary>
        /// Tears down configuration manager.
        /// </summary>
        [TearDown]
        public void TearDown()
        {
            manager.Clear();
        }

        /// <summary>
        /// Tests the static property key name.
        /// </summary>
        [Test]
        public void TestStaticPropertyKey()
        {
            Assert.AreEqual("extensions", ExtensionMatcher.ExtensionsPropertyKey,
                "The property key name is wrong");
        }

        /// <summary>
        /// Tests constructor(string) is initialized with a single extension
        /// and Match() only returns true for the specified extension.
        /// </summary>
        [Test]
        public void TestConstructor1()
        {
            // txt (we use tXt to test the case-insensitivity aspect)
            matcher = new ExtensionMatcher("tXt");

            // .txt, .TxT should match
            Assert.IsTrue(matcher.Match(new FileInfo(SampleText)),
                "txt should match");
            Assert.IsTrue(matcher.Match(new FileInfo("test.TxT")),
                "TxT should match (case-insensitive)");

            // should not match .A-.Z
            for (char ch = 'A'; ch <= 'Z'; ch++)
            {
                Assert.IsFalse(matcher.Match(new FileInfo("test." + ch)),
                    ch + " should not match");
            }

            // should not match file without extension
            Assert.IsFalse(matcher.Match(new FileInfo("test")),
                "File without extension should not match");
        }

        /// <summary>
        /// Tests constructor(string) is initialized with an empty extension
        /// and Match() returns true only for file with no extension.
        /// </summary>
        [Test]
        public void TestConstructor1Empty()
        {
            // no extension
            matcher = new ExtensionMatcher("");

            // .txt, .TXT should not match
            Assert.IsFalse(matcher.Match(new FileInfo(SampleText)),
                "txt should not match");
            Assert.IsFalse(matcher.Match(new FileInfo("test.TxT")),
                "TxT should not match (case-insensitive)");

            // should not match .A-.Z
            for (char ch = 'A'; ch <= 'Z'; ch++)
            {
                Assert.IsFalse(matcher.Match(new FileInfo("test." + ch)),
                    ch + " should not match");
            }

            // should match file without extension
            Assert.IsTrue(matcher.Match(new FileInfo("test")),
                "File without extension should match");
        }

        /// <summary>
        /// Tests constructor(string) throws ArgumentNullException
        /// given null argument.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor1Null()
        {
            new ExtensionMatcher((string) null);
        }

        /// <summary>
        /// Tests constructor(IList) is initialized with two extensions
        /// and Match() only returns true for the specified extensions.
        /// </summary>
        [Test]
        public void TestConstructor2()
        {
            // txt and c
            matcher = new ExtensionMatcher(exts);

            // .txt, .TxT, and .c should match
            Assert.IsTrue(matcher.Match(new FileInfo(SampleText)),
                "txt should match");
            Assert.IsTrue(matcher.Match(new FileInfo("test.TxT")),
                "TxT should match (case-insensitive)");
            Assert.IsTrue(matcher.Match(new FileInfo("test.c")),
                "c should match");

            // should not match .A-.Z, except .C
            for (char ch = 'A'; ch <= 'Z'; ch++)
            {
                // only .C should match
                if (ch == 'C')
                {
                    Assert.IsTrue(matcher.Match(new FileInfo("test." + ch)),
                        ch + " should match");
                }
                else
                {
                    Assert.IsFalse(matcher.Match(new FileInfo("test." + ch)),
                        ch + " should not match");
                }
            }

            // should not match file without extension
            Assert.IsFalse(matcher.Match(new FileInfo("test")),
                "File without extension should not match");
        }

        /// <summary>
        /// Tests constructor(IList) is initialized with an empty extension
        /// and Match() returns true only for the specified extensions
        /// and file without extension.
        /// </summary>
        [Test]
        public void TestConstructor2Empty()
        {
            // no extension is ok
            exts.Add("");
            matcher = new ExtensionMatcher(exts);

            // .txt, .TxT, and .c should match
            Assert.IsTrue(matcher.Match(new FileInfo(SampleText)),
                "txt should match");
            Assert.IsTrue(matcher.Match(new FileInfo("test.TxT")),
                "TxT should match (case-insensitive)");
            Assert.IsTrue(matcher.Match(new FileInfo("test.c")),
                "c should match");

            // should not match .A-.Z, except .C
            for (char ch = 'A'; ch <= 'Z'; ch++)
            {
                // only .C should match
                if (ch == 'C')
                {
                    Assert.IsTrue(matcher.Match(new FileInfo("test." + ch)),
                        ch + " should match");
                }
                else
                {
                    Assert.IsFalse(matcher.Match(new FileInfo("test." + ch)),
                        ch + " should not match");
                }
            }

            // should match file without extension
            Assert.IsTrue(matcher.Match(new FileInfo("test")),
                "File without extension should match");
        }

        /// <summary>
        /// Tests constructor(IList) throws ArgumentNullException
        /// given null argument.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor2Null()
        {
            new ExtensionMatcher((IList) null);
        }

        /// <summary>
        /// Tests constructor(IList) throws ArgumentException
        /// given empty list.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor2List()
        {
            new ExtensionMatcher(new ArrayList());
        }

        /// <summary>
        /// Tests constructor(IList) throws ArgumentException
        /// given list containing null element.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor2NullElement()
        {
            exts.Add(null);
            new ExtensionMatcher(exts);
        }

        /// <summary>
        /// Tests constructor(IList) throws ArgumentException
        /// given list containing non-string element.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor2NonStringElement()
        {
            exts.Add(123);
            new ExtensionMatcher(exts);
        }

        /// <summary>
        /// Tests constructor(IList) throws ArgumentException
        /// given list containing duplicate element.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor2DuplicateElement()
        {
            exts.Add(exts[0]);
            new ExtensionMatcher(exts);
        }

        /// <summary>
        /// Tests constructor(string, string) is initialized with two extensions
        /// and Match() only returns true for the specified extensions.
        /// </summary>
        [Test]
        public void TestConstructor3()
        {
            // initialize ExtensionMatcher (only .txt and .sql)
            matcher = new ExtensionMatcher(ValidNamespace, "matcher.");

            // .txt, .TxT, and .SqL should match
            Assert.IsTrue(matcher.Match(new FileInfo(SampleText)),
                "txt should match");
            Assert.IsTrue(matcher.Match(new FileInfo("test.TxT")),
                "TxT should match (case-insensitive)");
            Assert.IsTrue(matcher.Match(new FileInfo("test.SqL")),
                "SqL should match");

            // should not match .A-.Z
            for (char ch = 'A'; ch <= 'Z'; ch++)
            {
                Assert.IsFalse(matcher.Match(new FileInfo("test." + ch)),
                    ch + " should not match");
            }

            // should not match file without extension
            Assert.IsFalse(matcher.Match(new FileInfo("test")),
                "File without extension should not match");
        }

        /// <summary>
        /// Tests constructor(string, string) is initialized with an empty extension
        /// and Match() returns true only for file without extension.
        /// </summary>
        [Test]
        public void TestConstructor3Empty()
        {
            // initialize ExtensionMatcher (only empty)
            matcher = new ExtensionMatcher(ValidNamespace, "matcher_empty.");

            // .txt, .TxT, and .c should not match
            Assert.IsFalse(matcher.Match(new FileInfo(SampleText)),
                "txt should not match");
            Assert.IsFalse(matcher.Match(new FileInfo("test.TxT")),
                "TxT should not match (case-insensitive)");
            Assert.IsFalse(matcher.Match(new FileInfo("test.c")),
                "c should not match");

            // should not match .A-.Z
            for (char ch = 'A'; ch <= 'Z'; ch++)
            {
                Assert.IsFalse(matcher.Match(new FileInfo("test." + ch)),
                    ch + " should not match");
            }

            // should match file without extension
            Assert.IsTrue(matcher.Match(new FileInfo("test")),
                "File without extension should match");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentNullException
        /// given null nameSpace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor3NullNamespace()
        {
            new ExtensionMatcher(null, "matcher.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentNullException
        /// given null prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor3NullPrefix()
        {
            new ExtensionMatcher(ValidNamespace, null);
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentException
        /// given empty namespace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor3EmptyNamespace()
        {
            new ExtensionMatcher(" ", "matcher.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given non-existing prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructor3NotExist()
        {
            new ExtensionMatcher(ValidNamespace, "notexists.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given non-existing empty prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructor3NotExistEmpty()
        {
            new ExtensionMatcher(ValidNamespace, "");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given non-existing empty-space prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructor3NotExistEmptySpace()
        {
            new ExtensionMatcher(ValidNamespace, " ");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given no list.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructor3NoList()
        {
            new ExtensionMatcher(ValidNamespace, "matcher_invalid.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given list containing duplicate element.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructor3DuplicateElement()
        {
            new ExtensionMatcher(ValidNamespace, "matcher_duplicate.");
        }

        /// <summary>
        /// Tests Match() throws ArgumentNullException
        /// given null FileInfo.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestMatchNull()
        {
            matcher.Match(null);
        }
    }
}
