/*
 * TestFileCategoryMatcher.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.IO;
using NUnit.Framework;
using TopCoder.File.Statistics.Tests;

namespace TopCoder.File.Statistics.Matcher.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="FileCategoryMatcher"/> abstract class.
    /// </summary>
    [TestFixture]
    public class TestFileCategoryMatcher : TestHelper
    {
        /// <summary>
        /// A concrete class inherited from <see cref="FileCategoryMatcher"/>
        /// which is used by <see cref="TestFileCategoryMatcher"/>
        /// to test <see cref="FileCategoryMatcher"/>.
        /// </summary>
        ///
        /// <remarks>
        /// This class only implements the abstract member Match
        /// by saving the passed boolean flag so that it can be asserted
        /// using the <see cref="FileIsBinary"/> property.
        /// </remarks>
        public class FileCategoryMatcherClass : FileCategoryMatcher
        {
            /// <summary>
            /// Initializes a new instance of the <see cref="FileCategoryMatcherClass"/> instance.
            /// </summary>
            public FileCategoryMatcherClass()
            {
            }

            /// <summary>
            /// The IsBinary flag passed to the template method.
            /// </summary>
            private bool fileIsBinary;

            /// <summary>
            /// Gets the IsBinary flag passed to the template method.
            /// </summary>
            ///
            /// <value>
            /// The IsBinary flag passed to the template method.
            /// </value>
            public bool FileIsBinary
            {
                get
                {
                    return fileIsBinary;
                }
            }

            /// <summary>
            /// Saves the flag so that it can be asserted.
            /// </summary>
            ///
            /// <param name="isBinary">
            /// The flag to save.
            /// </param>
            ///
            /// <returns>
            /// Always <b>true</b>.
            /// </returns>
            protected override bool Match(bool isBinary)
            {
                // save the flag
                this.fileIsBinary = isBinary;
                return true;
            }
        }

        /// <summary>
        /// The <see cref="FileCategoryMatcherClass"/> instance to test.
        /// Initialized in SetUp().
        /// </summary>
        private FileCategoryMatcherClass matcher;

        /// <summary>
        /// Initializes the <see cref="FileCategoryMatcherClass"/> instance to test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            matcher = new FileCategoryMatcherClass();
        }

        /// <summary>
        /// Tests constructor() doesn't throw any exception.
        /// </summary>
        [Test]
        public void TestConstructor1()
        {
            new FileCategoryMatcherClass();
        }

        /// <summary>
        /// Tests IsBinary with a binary file.
        /// </summary>
        [Test]
        public void TestIsBinaryBinary()
        {
            // use test_files/Matcher_FileCategory_Binary.bin
            Assert.IsTrue(
                FileCategoryMatcher.IsBinary(new FileInfo(@"..\..\test_files\Matcher_FileCategory_Binary.bin")),
                "The file should be detected as a binary");
        }

        /// <summary>
        /// Tests IsBinary with a text file.
        /// </summary>
        [Test]
        public void TestIsBinaryText()
        {
            // use test_files/Matcher_FileCategory_Text.txt
            Assert.IsFalse(
                FileCategoryMatcher.IsBinary(new FileInfo(@"..\..\test_files\Matcher_FileCategory_Text.txt")),
                "The file should be detected as a text");
        }

        /// <summary>
        /// Tests IsBinary with an empty file.
        /// </summary>
        [Test]
        public void TestIsBinaryEmpty()
        {
            // use test_files/Matcher_FileCategory_Empty
            Assert.IsFalse(
                FileCategoryMatcher.IsBinary(new FileInfo(@"..\..\test_files\Matcher_FileCategory_Empty")),
                "Empty file is considered a text");
        }

        /// <summary>
        /// Tests IsBinary throws ArgumentNullException
        /// given null argument.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestIsBinaryNull()
        {
            FileCategoryMatcher.IsBinary(null);
        }

        /// <summary>
        /// Tests IsBinary throws IOException
        /// given non-existing file.
        /// </summary>
        [Test]
        public void TestIsBinaryNotExist()
        {
            try
            {
                // this may throw FileNotFoundException, which is a subclass of IOException
                FileCategoryMatcher.IsBinary(new FileInfo("notexists"));
                Assert.Fail("IOException expected");
            }
            catch (IOException)
            {
                // ok
            }
        }

        /// <summary>
        /// Tests IsBinary throws IOException
        /// given a directory.
        /// </summary>
        [Test]
        [ExpectedException(typeof(IOException))]
        public void TestIsBinaryDirectory()
        {
            FileCategoryMatcher.IsBinary(new FileInfo(@"..\..\test_files"));
        }

        /// <summary>
        /// Tests Match(FileInfo) with a binary file.
        /// </summary>
        [Test]
        public void TestMatchBinary()
        {
            // a binary file
            matcher.Match(new FileInfo(@"..\..\test_files\Matcher_FileCategory_Binary.bin"));

            // assert the IsBinary flag
            Assert.IsTrue(matcher.FileIsBinary,
                "The file should be detected as a binary");
        }

        /// <summary>
        /// Tests Match(FileInfo) with a text file.
        /// </summary>
        [Test]
        public void TestMatchText()
        {
            // a text file
            matcher.Match(new FileInfo(@"..\..\test_files\Matcher_FileCategory_Text.txt"));

            // assert the IsBinary flag
            Assert.IsFalse(matcher.FileIsBinary,
                "The file should be detected as a text");
        }

        /// <summary>
        /// Tests Match() throws IOException
        /// given a non-existing file.
        /// </summary>
        [Test]
        public void TestMatchNotExist()
        {
            try
            {
                // the file doesn't exist
                // this may throw FileNotFoundException, which is a subclass of IOException
                matcher.Match(new FileInfo(@"..\..\test_files\notexists.txt"));
                Assert.Fail("IOException expected");
            }
            catch (IOException)
            {
                // ok
            }
        }

        /// <summary>
        /// Tests Match() throws IOException
        /// given an existing directory.
        /// </summary>
        [Test]
        [ExpectedException(typeof(IOException))]
        public void TestMatchDirectory()
        {
            // an existing directory
            matcher.Match(new FileInfo(@"..\..\test_files"));
        }

        /// <summary>
        /// Tests Match() throws ArgumentNullException
        /// given null FileInfo.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestMatchNull()
        {
            matcher.Match(null);
        }
    }
}
