/*
 * TestFileProcessorBase.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.Collections;
using System.IO;
using NUnit.Framework;
using TopCoder.File.Statistics.Matcher;
using TopCoder.File.Statistics.Tests;

namespace TopCoder.File.Statistics.Processor.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="FileProcessorBase"/> abstract class.
    /// </summary>
    [TestFixture]
    public class TestFileProcessorBase : TestHelper
    {
        /// <summary>
        /// A concrete class inherited from <see cref="FileProcessorBase"/>
        /// which is used by <see cref="TestFileProcessorBase"/>
        /// to test <see cref="FileProcessorBase"/>.
        /// </summary>
        public class FileProcessorBaseClass : FileProcessorBase
        {
            /// <summary>
            /// Gets a list of all the supported metrics by this processor.
            /// </summary>
            ///
            /// <value>
            /// Always null.
            /// </value>
            public override IList SupportedMetrics
            {
                get
                {
                    return null;
                }
            }

            /// <summary>
            /// Initializes a new instance of the <see cref="FileProcessorBaseClass"/> instance.
            /// </summary>
            ///
            /// <param name="alias">
            /// The alias name.
            /// </param>
            /// <param name="matcher">
            /// The <see cref="IFileMatcher"/> to use.
            /// </param>
            ///
            /// <exception cref="ArgumentNullException">
            /// Any of the argument is a null reference.
            /// </exception>
            /// <exception cref="ArgumentException">
            /// <paramref name="alias"/> is empty after trimmed.
            /// </exception>
            public FileProcessorBaseClass(string alias, IFileMatcher matcher)
                : base(alias, matcher)
            {
            }

            /// <summary>
            /// Initializes a new instance of the <see cref="FileProcessorBaseClass"/>
            /// which loads configuration from the specified nameSpace
            /// and prefix of the configuration.
            /// </summary>
            ///
            /// <remarks>
            /// The alias name will be loaded from the property <c>prefix + "alias"</c>,
            /// while the file matcher class will be loaded from the property
            /// <c>prefix + "matcher.classname"</c> passing the nameSpace and
            /// <c>prefix + "matcher."</c> to the file matcher constructor.
            /// If the property <c>prefix + "matcher.classname" is missing,
            /// <see cref="AnyFileMatcher"/> will be used.</c>
            /// </remarks>
            ///
            /// <param name="nameSpace">
            /// The namespace to load configuration values from.
            /// </param>
            /// <param name="prefix">
            /// The prefix of the property name to load configuration values from.
            /// </param>
            ///
            /// <exception cref="ArgumentNullException">
            /// Any of the argument is a null reference.
            /// </exception>
            /// <exception cref="ArgumentException">
            /// Any of the argument is empty after trimmed.
            /// </exception>
            /// <exception cref="ConfigurationException">
            /// A configuration error occurs, such as missing alias name,
            /// missing empty alias name, error instantiating file matcher.
            /// </exception>
            public FileProcessorBaseClass(string nameSpace, string prefix)
                : base(nameSpace, prefix)
            {
            }

            /// <summary>
            /// Processes the file and generates the statistic result.
            /// </summary>
            ///
            /// <param name="file">
            /// N/A.
            /// </param>
            ///
            /// <returns>
            /// Always null.
            /// </returns>
            public override StatsResult Process(FileInfo file)
            {
                return null;
            }
        }

        /// <summary>
        /// The <see cref="FileProcessorBaseClass"/> instance to test.
        /// Initialized in SetUp().
        /// </summary>
        private IFileProcessor processor;

        /// <summary>
        /// Initializes the <see cref="FileProcessorBaseClass"/> instance to test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            // load test_files/Processor_Base.xml
            // used for constructor with nameSpace and prefix argument
            base.LoadConfigFile(@"..\..\test_files\Processor_Base.xml");

            // initialize with ExtensionMatcher
            processor = new FileProcessorBaseClass(ValidNamespace, "processor1.");
        }

        /// <summary>
        /// Tests the static properties.
        /// </summary>
        [Test]
        public void TestStaticProperties()
        {
            Assert.AreEqual("alias", FileProcessorBase.AliasPropertyKey,
                "The alias property key is wrong");
            Assert.AreEqual("matcher", FileProcessorBase.FileMatcherPropertyKey,
                "The matcher property key is wrong");
            Assert.AreEqual("classname", FileProcessorBase.ClassnamePropertyKey,
                "The classname property key is wrong");
        }

        /// <summary>
        /// Tests constructor(string, string) loads alias and file matcher correcly.
        /// </summary>
        [Test]
        public void TestConstructor()
        {
            Assert.AreEqual("Processor1 alias", processor.Alias,
                "The processor alias is not correctly loaded");
            Assert.IsTrue(processor.FileMatcher is ExtensionMatcher,
                "The file matcher is not correctly loaded");

            // assert the matcher can match only txt and sql
            Assert.IsTrue(processor.FileMatcher.Match(new FileInfo("test.txt")),
                "txt should match");
            Assert.IsTrue(processor.FileMatcher.Match(new FileInfo("test.sql")),
                "sql should match");
            Assert.IsFalse(processor.FileMatcher.Match(new FileInfo("test.bin")),
                "bin should not match");
        }

        /// <summary>
        /// Tests constructor(string, string) loads alias and AnyFileMatcher
        /// if no matcher is specified.
        /// </summary>
        [Test]
        public void TestConstructorAnyFileMatcher()
        {
            // use the nomatcher prefix
            processor = new FileProcessorBaseClass(ValidNamespace, "nomatcher.");

            // assert the alias and AnyFileMatcher
            Assert.AreEqual("Processor2 with AnyFileMatcher", processor.Alias,
                "The processor alias is not correctly loaded");
            Assert.IsTrue(processor.FileMatcher is AnyFileMatcher,
                "AnyFileMatcher should be loaded");

            // assert the matcher can match all
            Assert.IsTrue(processor.FileMatcher.Match(new FileInfo("test.txt")),
                "txt should match");
            Assert.IsTrue(processor.FileMatcher.Match(new FileInfo("test.sql")),
                "sql should match");
            Assert.IsTrue(processor.FileMatcher.Match(new FileInfo("test.bin")),
                "bin should not match");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentNullException
        /// given null namespace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructorNullNamespace()
        {
            new FileProcessorBaseClass(null, "processor1.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentNullException
        /// given null prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructorNullPrefix()
        {
            new FileProcessorBaseClass(ValidNamespace, (string) null);
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentException
        /// given empty namespace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructorEmptyNamespace()
        {
            new FileProcessorBaseClass(" ", "processor1.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given non-existing prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructorNotExist()
        {
            new FileProcessorBaseClass(ValidNamespace, "notexists.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given non-existing empty prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructorNotExistEmpty()
        {
            new FileProcessorBaseClass(ValidNamespace, "");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given non-existing empty-space prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructorNotExistEmptySpace()
        {
            new FileProcessorBaseClass(ValidNamespace, " ");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given no alias.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructorNoAlias()
        {
            new FileProcessorBaseClass(ValidNamespace, "noalias.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given empty alias.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructorEmptyAlias()
        {
            new FileProcessorBaseClass(ValidNamespace, "emptyalias.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given matcher that doesn't exist.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructorNotExistMatcher()
        {
            new FileProcessorBaseClass(ValidNamespace, "invalidmatcher.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given matcher that is not IFileMatcher.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructorInvalidMatcher()
        {
            new FileProcessorBaseClass(ValidNamespace, "invalidmatcher2.");
        }

        /// <summary>
        /// Tests constructor(string, IFileMatcher) accuracy.
        /// </summary>
        [Test]
        public void TestConstructor2()
        {
            // instantiate a processor
            processor = new FileProcessorBaseClass("the alias", new ExtensionMatcher("txt"));

            // assert
            Assert.AreEqual("the alias", processor.Alias,
                "The processor alias is not correctly initialized");
            Assert.IsTrue(processor.FileMatcher is ExtensionMatcher,
                "The file matcher is not correctly initialized");

            // assert the matcher can match only txt
            Assert.IsTrue(processor.FileMatcher.Match(new FileInfo("test.txt")),
                "txt should match");
            Assert.IsFalse(processor.FileMatcher.Match(new FileInfo("test.sql")),
                "sql should match");
            Assert.IsFalse(processor.FileMatcher.Match(new FileInfo("test.bin")),
                "bin should not match");
        }

        /// <summary>
        /// Tests constructor(string, IFileMatcher) throws ArgumentNullException
        /// given null namespace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor2NullNamespace()
        {
            new FileProcessorBaseClass(null, new AnyFileMatcher());
        }

        /// <summary>
        /// Tests constructor(string, IFileMatcher) throws ArgumentNullException
        /// given null file matcher.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor2NullMatcher()
        {
            new FileProcessorBaseClass(ValidNamespace, (IFileMatcher) null);
        }

        /// <summary>
        /// Tests constructor(string, IFileMatcher) throws ArgumentException
        /// given empty namespace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor2EmptyNamespace()
        {
            new FileProcessorBaseClass(" ", new AnyFileMatcher());
        }
    }
}
