/*
 * TestAbstractStatsReporting.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.Collections;
using System.IO;
using NUnit.Framework;
using TopCoder.File.Statistics.Matcher;
using TopCoder.File.Statistics.Processor.LineCounter;
using TopCoder.File.Statistics.Tests;

namespace TopCoder.File.Statistics.Reporting.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="AbstractStatsReporting"/> abstract class.
    /// </summary>
    [TestFixture]
    public class TestAbstractStatsReporting : TestHelper
    {
        /// <summary>
        /// A concrete class inherited from <see cref="AbstractStatsReporting"/>
        /// which is used by <see cref="TestAbstractStatsReporting"/>
        /// to test <see cref="AbstractStatsReporting"/>.
        /// </summary>
        public class AbstractStatsReportingClass : AbstractStatsReporting
        {
            /// <summary>
            /// Initializes a new instance of the <see cref="AbstractStatsReportingClass"/> class.
            /// </summary>
            public AbstractStatsReportingClass()
            {
            }

            /// <summary>
            /// Initializes a new instance of the <see cref="AbstractStatsReportingClass"/>
            /// which loads configuration from the specified nameSpace
            /// and prefix of the configuration.
            /// </summary>
            ///
            /// <remarks>
            /// This constructor is provided for extension purpose only.
            /// No configuration is processed by this constructor.
            /// </remarks>
            ///
            /// <param name="nameSpace">
            /// The namespace to load configuration values from.
            /// </param>
            /// <param name="prefix">
            /// The prefix of the property name to load configuration values from.
            /// </param>
            ///
            /// <exception cref="ArgumentNullException">
            /// Any of the argument is a null reference.
            /// </exception>
            /// <exception cref="ArgumentException">
            /// Any of the argument is empty after trimmed.
            /// </exception>
            /// <exception cref="ConfigurationException">
            /// A configuration error occurs.
            /// </exception>
            public AbstractStatsReportingClass(string nameSpace, string prefix)
                : base(nameSpace, prefix)
            {
            }

            /// <summary>
            /// Generates the report with all the aggregated results.
            /// </summary>
            ///
            /// <returns>
            /// Always <b>null</b>.
            /// </returns>
            public override string GenerateReport()
            {
                return null;
            }
        }

        /// <summary>
        /// A sample Metric name.
        /// </summary>
        private const string name = "Word count";

        /// <summary>
        /// A sample Metric display name.
        /// </summary>
        private const string displayName = "wcount";

        /// <summary>
        /// A sample Metric description.
        /// </summary>
        private const string description = "Count the number of words";

        /// <summary>
        /// The processor alias name.
        /// We'll test with 2 processor alias, composed of alias + "1"/"2".
        /// See SetUp().
        /// </summary>
        private const string alias = "Processor alias";

        /// <summary>
        /// A sample initial metric to be stored in StatsResult.
        /// Initialized in SetUp().
        /// </summary>
        private Metric metric;

        /// <summary>
        /// A sample statistic result for a file.
        /// Initialized in SetUp().
        /// </summary>
        private StatsResult result;

        /// <summary>
        /// The IFileProcessor which generates the result.
        /// Only the alias name is important for the report.
        /// We'll create 2 processor instances with the name alias + "1"/"2".
        /// Initialized to TextLineCounter() in SetUp().
        /// </summary>
        private IFileProcessor[] processor;

        /// <summary>
        /// The <see cref="AbstractStatsReportingClass"/> instance to test.
        /// Initialized in SetUp() with 1 sample result.
        /// </summary>
        private IStatsReporting report;

        /// <summary>
        /// Initializes the <see cref="AbstractStatsReportingClass"/> instance to test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            // initialize the IFileProcessor
            // only the alias name is important
            processor = new IFileProcessor[]
                {
                    new TextLineCounter(alias + "1", new AnyFileMatcher()),
                    new TextLineCounter(alias + "2", new AnyFileMatcher())
                };

            // create a StatsResult containing a metric with an initial value of 10
            metric = new Metric(name, displayName, description);
            result = new StatsResult();
            result[metric] = 10;

            // initialize the AbstractStatsReportingClass
            // reset to test_files folder
            // and aggregate a result
            report = new AbstractStatsReportingClass();
            report.Reset(new DirectoryInfo(@"..\..\test_files"));
            report.Aggregate(processor[0], new FileInfo(SampleText), result);
        }

        /// <summary>
        /// Tests BaseDirectory property getter.
        /// </summary>
        [Test]
        public void TestBaseDirectory()
        {
            DirectoryInfo baseDir = ((AbstractStatsReporting) report).BaseDirectory;

            Assert.AreEqual("test_files", baseDir.Name,
                "The base directory is test_files");
            Assert.IsTrue(baseDir.FullName.EndsWith("test_files"),
                "The base directory fullname ends with test_files");
        }

        /// <summary>
        /// Tests BaseDirectory if Reset is called with a DirectoryInfo
        /// which ends with a backslash.
        /// </summary>
        [Test]
        public void TestBaseDirectoryBackslash()
        {
            // the DirectoryInfo ends with a backslash.
            report.Reset(new DirectoryInfo(@"..\..\test_files\"));
            DirectoryInfo baseDir = ((AbstractStatsReporting) report).BaseDirectory;

            Assert.AreEqual("test_files", baseDir.Name,
                "The base directory is test_files");
            Assert.IsTrue(baseDir.FullName.EndsWith(@"test_files\"),
                @"The base directory fullname ends with test_files\");
        }

        /// <summary>
        /// Tests FileStats property getter.
        /// </summary>
        [Test]
        public void TestFileStats()
        {
            StatsCollection stats = ((AbstractStatsReporting) report).FileStats;

            // assert 1 stats
            IList keys = stats.AllKeys;
            Assert.AreEqual(1, keys.Count, "1 key is stored");
            Assert.AreEqual(@"test\test.txt", keys[0], "The key name is relative");

            // assert the StatsResult
            StatsResult result = stats[(string) keys[0]];
            // 1 metric is stored
            IList metrics = result.Metrics;
            Assert.AreEqual(1, metrics.Count, "1 metric is stored");

            // assert the metric
            Metric m = metrics[0] as Metric;
            Assert.AreEqual(metric, m, "The Metric is incorrect");
            Assert.AreEqual(10, result[m], "The metric value should be 10");
        }

        /// <summary>
        /// Tests DirectoryStats property getter.
        /// </summary>
        [Test]
        public void TestDirectoryStats()
        {
            StatsCollection stats = ((AbstractStatsReporting) report).DirectoryStats;

            // assert 1 stats
            IList keys = stats.AllKeys;
            Assert.AreEqual(1, keys.Count, "1 key is stored");
            Assert.AreEqual(@"test", keys[0], "The key name is relative");

            // assert the StatsResult
            StatsResult result = stats[(string) keys[0]];
            // 1 metric is stored
            IList metrics = result.Metrics;
            Assert.AreEqual(1, metrics.Count, "1 metric is stored");

            // assert the metric
            Metric m = metrics[0] as Metric;
            Assert.AreEqual(metric, m, "The Metric is incorrect");
            Assert.AreEqual(10, result[m], "The metric value should be 10");
        }

        /// <summary>
        /// Tests AliasStats property getter.
        /// </summary>
        [Test]
        public void TestAliasStats()
        {
            StatsCollection stats = ((AbstractStatsReporting) report).AliasStats;

            // assert 1 stats
            IList keys = stats.AllKeys;
            Assert.AreEqual(1, keys.Count, "1 key is stored");
            Assert.AreEqual(processor[0].Alias, keys[0], "The key name is the alias");

            // assert the StatsResult
            StatsResult result = stats[(string) keys[0]];
            // 1 metric is stored
            IList metrics = result.Metrics;
            Assert.AreEqual(1, metrics.Count, "1 metric is stored");

            // assert the metric
            Metric m = metrics[0] as Metric;
            Assert.AreEqual(metric, m, "The Metric is incorrect");
            Assert.AreEqual(10, result[m], "The metric value should be 10");
        }

        /// <summary>
        /// Tests OverallStats property getter.
        /// </summary>
        [Test]
        public void TestOverallStats()
        {
            StatsResult result = ((AbstractStatsReporting) report).OverallStats;

            // assert the StatsResult
            // 1 metric is stored
            IList metrics = result.Metrics;
            Assert.AreEqual(1, metrics.Count, "1 metric is stored");

            // assert the metric
            Metric m = metrics[0] as Metric;
            Assert.AreEqual(metric, m, "The Metric is incorrect");
            Assert.AreEqual(10, result[m], "The metric value should be 10");
        }

        /// <summary>
        /// Tests Aggregate a new StatsResult
        /// and tests FileStats property getter
        /// </summary>
        [Test]
        public void TestFileStatsAdd()
        {
            // add a different StatsResult for processor[1]
            result[metric] = 7;
            report.Aggregate(processor[1], new FileInfo(@"..\..\test_files\test\aaa\sample.txt"), result);

            StatsCollection stats = ((AbstractStatsReporting) report).FileStats;

            // assert 2 stats (sorted by filename)
            IList keys = stats.AllKeys;
            Assert.AreEqual(2, keys.Count, "2 keys are stored");
            Assert.AreEqual(@"test\aaa\sample.txt", keys[0], "The 1st key name is relative");
            Assert.AreEqual(@"test\test.txt", keys[1], "The 2nd key name is relative");

            // assert the 1st StatsResult
            result = stats[(string) keys[0]];
            // 1 metric is stored
            IList metrics = result.Metrics;
            Assert.AreEqual(1, metrics.Count, "1 metric is stored in 1st stat");
            // assert the metric
            Metric m = metrics[0] as Metric;
            Assert.AreEqual(metric, m, "The 1st stat metric is incorrect");
            Assert.AreEqual(7, result[m], "The 1st metric value should be 7");

            // assert the 2nd StatsResult
            result = stats[(string) keys[1]];
            // 1 metric is stored
            metrics = result.Metrics;
            Assert.AreEqual(1, metrics.Count, "1 metric is stored in 2nd stat");
            // assert the metric
            m = metrics[0] as Metric;
            Assert.AreEqual(metric, m, "The 2nd stat metric is incorrect");
            Assert.AreEqual(10, result[m], "The 2nd metric value should be 10");
        }

        /// <summary>
        /// Tests Aggregate a new StatsResult
        /// and tests DirectoryStats property getter
        /// </summary>
        [Test]
        public void TestDirectoryStatsAdd()
        {
            // add a different StatsResult for processor[1]
            result[metric] = 7;
            report.Aggregate(processor[1], new FileInfo(@"..\..\test_files\test\aaa\sample.txt"), result);

            StatsCollection stats = ((AbstractStatsReporting) report).DirectoryStats;

            // assert 2 stats (sorted by dirname)
            IList keys = stats.AllKeys;
            Assert.AreEqual(2, keys.Count, "2 keys are stored");
            Assert.AreEqual(@"test", keys[0], "The 1st key name is relative");
            Assert.AreEqual(@"test\aaa", keys[1], "The 2nd key name is relative");

            // assert the 1st StatsResult
            result = stats[(string) keys[0]];
            // 1 metric is stored
            IList metrics = result.Metrics;
            Assert.AreEqual(1, metrics.Count, "1 metric is stored in 1st stat");
            // assert the metric
            Metric m = metrics[0] as Metric;
            Assert.AreEqual(metric, m, "The 1st stat metric is incorrect");
            Assert.AreEqual(10 + 7, result[m], "The 1st metric value should be 17");

            // assert the 2nd StatsResult
            result = stats[(string) keys[1]];
            // 1 metric is stored
            metrics = result.Metrics;
            Assert.AreEqual(1, metrics.Count, "1 metric is stored in 2nd stat");
            // assert the metric
            m = metrics[0] as Metric;
            Assert.AreEqual(metric, m, "The 2nd stat metric is incorrect");
            Assert.AreEqual(7, result[m], "The 2nd metric value should be 7");
        }

        /// <summary>
        /// Tests Aggregate a new StatsResult
        /// and tests AliasStats property getter
        /// </summary>
        [Test]
        public void TestAliasStatsAdd()
        {
            // add a different StatsResult for processor[1]
            result[metric] = 7;
            report.Aggregate(processor[1], new FileInfo(@"..\..\test_files\test\aaa\sample.txt"), result);

            StatsCollection stats = ((AbstractStatsReporting) report).AliasStats;

            // assert 2 stats (sorted by alias name)
            IList keys = stats.AllKeys;
            Assert.AreEqual(2, keys.Count, "2 keys are stored");
            Assert.AreEqual(processor[0].Alias, keys[0], "The 1st alias");
            Assert.AreEqual(processor[1].Alias, keys[1], "The 2nd alias");

            // assert the 1st StatsResult
            result = stats[(string) keys[0]];
            // 1 metric is stored
            IList metrics = result.Metrics;
            Assert.AreEqual(1, metrics.Count, "1 metric is stored in 1st stat");
            // assert the metric
            Metric m = metrics[0] as Metric;
            Assert.AreEqual(metric, m, "The 1st stat metric is incorrect");
            Assert.AreEqual(10, result[m], "The 1st metric value should be 10");

            // assert the 2nd StatsResult
            result = stats[(string) keys[1]];
            // 1 metric is stored
            metrics = result.Metrics;
            Assert.AreEqual(1, metrics.Count, "1 metric is stored in 2nd stat");
            // assert the metric
            m = metrics[0] as Metric;
            Assert.AreEqual(metric, m, "The 2nd stat metric is incorrect");
            Assert.AreEqual(7, result[m], "The 2nd metric value should be 7");
        }

        /// <summary>
        /// Tests OverallStats property getter.
        /// </summary>
        [Test]
        public void TestOverallStatsAdd()
        {
            // add a different StatsResult for processor[1]
            result[metric] = 7;
            report.Aggregate(processor[1], new FileInfo(@"..\..\test_files\test\aaa\sample.txt"), result);

            result = ((AbstractStatsReporting) report).OverallStats;

            // assert the StatsResult
            // 1 metric is stored
            IList metrics = result.Metrics;
            Assert.AreEqual(1, metrics.Count, "1 metric is stored");

            // assert the metric
            Metric m = metrics[0] as Metric;
            Assert.AreEqual(metric, m, "The Metric is incorrect");
            Assert.AreEqual(10 + 7, result[m], "The metric value should be 17");
        }

        /// <summary>
        /// Tests Reset clears all statistics.
        /// </summary>
        [Test]
        public void TestReset()
        {
            // clears and sets a new base
            report.Reset(new DirectoryInfo(@"..\..\test_files\test"));

            // assert all returns 0
            AbstractStatsReporting rpt = (AbstractStatsReporting) report;
            Assert.AreEqual(0, rpt.FileStats.AllKeys.Count, "FileStats cleared");
            Assert.AreEqual(0, rpt.DirectoryStats.AllKeys.Count, "DirectoryStats cleared");
            Assert.AreEqual(0, rpt.AliasStats.AllKeys.Count, "AliasStats cleared");
            Assert.AreEqual(0, rpt.OverallStats.Metrics.Count, "OverallStats cleared");

            // base directory changed
            Assert.AreEqual("test",
                ((AbstractStatsReporting) report).BaseDirectory.Name,
                "The base directory is test");
        }

        /// <summary>
        /// Tests FileStats with null BaseDirectory.
        /// </summary>
        [Test]
        public void TestFileStatsNullBase()
        {
            // reset BaseDirectory to null and add a StatsResult
            report.Reset(null);
            report.Aggregate(processor[0], new FileInfo(@"..\..\test_files\test\aaa\sample.txt"), result);

            StatsCollection stats = ((AbstractStatsReporting) report).FileStats;

            // assert 1 stats
            IList keys = stats.AllKeys;
            Assert.AreEqual(1, keys.Count, "1 key is stored");
            Assert.IsTrue(((string) keys[0]).EndsWith(@"\test_files\test\aaa\sample.txt"),
                "The key name is absolute");

            // assert the StatsResult
            result = stats[(string) keys[0]];
            // 1 metric is stored
            IList metrics = result.Metrics;
            Assert.AreEqual(1, metrics.Count, "1 metric is stored");

            // assert the metric
            Metric m = metrics[0] as Metric;
            Assert.AreEqual(metric, m, "The Metric is incorrect");
            Assert.AreEqual(10, result[m], "The metric value should be 10");
        }

        /// <summary>
        /// Tests DirectoryStats with null BaseDirectory.
        /// </summary>
        [Test]
        public void TestDirectoryStatsNullBase()
        {
            // reset BaseDirectory to null and add a StatsResult
            report.Reset(null);
            report.Aggregate(processor[0], new FileInfo(@"..\..\test_files\test\aaa\sample.txt"), result);

            StatsCollection stats = ((AbstractStatsReporting) report).DirectoryStats;

            // assert at least 3 stats (depending on dir depth up to root)
            IList keys = stats.AllKeys;
            Assert.IsTrue(keys.Count > 3, "At least 3 keys are stored");
            int count = keys.Count;
            Assert.IsTrue(((string) keys[--count]).EndsWith(@"\test_files\test\aaa"),
                "The last key name is absolute");
            Assert.IsTrue(((string) keys[--count]).EndsWith(@"\test_files\test"),
                "The 2nd last key name is absolute");
            Assert.IsTrue(((string) keys[--count]).EndsWith(@"\test_files"),
                "The 3rd last key name is absolute");

            // they all have the same StatsResult
            for (int i = 0; i < keys.Count; i++)
            {
                // make sure keys[i] is a subfolder of key[i - 1] where i > 0
                if (i > 0)
                {
                    Assert.IsTrue(((string) keys[i]).StartsWith((string) keys[i - 1]),
                        string.Format("keys[i] '{0}' should be a subfolder of key[i - 1] '{1}'",
                        keys[i], keys[i - 1]));
                }

                // assert the StatsResult
                result = stats[(string) keys[i]];
                // 1 metric is stored
                IList metrics = result.Metrics;
                Assert.AreEqual(1, metrics.Count, "1 metric is stored");

                // assert the metric
                Metric m = metrics[0] as Metric;
                Assert.AreEqual(metric, m, "The Metric is incorrect");
                Assert.AreEqual(10, result[m], "The metric value should be 10");
            }
        }

        /// <summary>
        /// Tests AliasStats with null BaseDirectory.
        /// </summary>
        [Test]
        public void TestAliasStatsNullBase()
        {
            // reset BaseDirectory to null and add a StatsResult
            report.Reset(null);
            report.Aggregate(processor[0], new FileInfo(@"..\..\test_files\test\aaa\sample.txt"), result);

            StatsCollection stats = ((AbstractStatsReporting) report).AliasStats;

            // assert 1 stats
            IList keys = stats.AllKeys;
            Assert.AreEqual(1, keys.Count, "1 key is stored");
            Assert.AreEqual(processor[0].Alias, keys[0], "The key name is the alias");

            // assert the StatsResult
            result = stats[(string) keys[0]];
            // 1 metric is stored
            IList metrics = result.Metrics;
            Assert.AreEqual(1, metrics.Count, "1 metric is stored");

            // assert the metric
            Metric m = metrics[0] as Metric;
            Assert.AreEqual(metric, m, "The Metric is incorrect");
            Assert.AreEqual(10, result[m], "The metric value should be 10");
        }

        /// <summary>
        /// Tests OverallStats with null BaseDirectory.
        /// </summary>
        [Test]
        public void TestOverallStatsNullBase()
        {
            // reset BaseDirectory to null and add a StatsResult
            report.Reset(null);
            report.Aggregate(processor[0], new FileInfo(@"..\..\test_files\test\aaa\sample.txt"), result);

            result = ((AbstractStatsReporting) report).OverallStats;

            // assert the StatsResult
            // 1 metric is stored
            IList metrics = result.Metrics;
            Assert.AreEqual(1, metrics.Count, "1 metric is stored");

            // assert the metric
            Metric m = metrics[0] as Metric;
            Assert.AreEqual(metric, m, "The Metric is incorrect");
            Assert.AreEqual(10, result[m], "The metric value should be 10");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentNullException
        /// given null namespace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructorNullNamespace()
        {
            new AbstractStatsReportingClass(null, "processor1.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentNullException
        /// given null prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructorNullPrefix()
        {
            new AbstractStatsReportingClass(ValidNamespace, (string) null);
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentException
        /// given empty namespace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructorEmptyNamespace()
        {
            new AbstractStatsReportingClass(" ", "processor1.");
        }

        /// <summary>
        /// Tests Aggregate throws ArgumentNullException
        /// given null processor.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestAggregateNullProcessor()
        {
            report.Aggregate(null, new FileInfo(SampleText), result);
        }

        /// <summary>
        /// Tests Aggregate throws ArgumentNullException
        /// given null file.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestAggregateNullFile()
        {
            report.Aggregate(processor[0], null, result);
        }

        /// <summary>
        /// Tests Aggregate throws ArgumentNullException
        /// given null result.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestAggregateNullResult()
        {
            report.Aggregate(processor[0], new FileInfo(SampleText), null);
        }

        /// <summary>
        /// Tests Aggregate throws ArgumentException
        /// given a file not in base-dir.
        /// </summary>
        [Test]
        [ExpectedException(typeof(StatsReportingException))]
        public void TestAggregateNotInBase()
        {
            // change the base to test_files\test
            report.Reset(new DirectoryInfo(@"..\..\test_files\test"));

            // add test_files\FileStatistic.xml
            report.Aggregate(processor[0], new FileInfo(ValidConfig), result);
        }

        /// <summary>
        /// Tests Aggregate throws ArgumentException
        /// given a file (with similar name with the BaseDir) not in base-dir.
        /// </summary>
        [Test]
        [ExpectedException(typeof(StatsReportingException))]
        public void TestAggregateNotInBase2()
        {
            // change the base to test_files\test
            report.Reset(new DirectoryInfo(@"..\..\test_files\test"));

            // add test_files\test.txt (this is not in test_files\test)
            report.Aggregate(processor[0], new FileInfo(@"..\..\test_files\test.txt"), result);
        }
    }
}
