/*
 * TestXmlStatsReporting.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.Collections;
using System.IO;
using NUnit.Framework;
using TopCoder.File.Statistics.Matcher;
using TopCoder.File.Statistics.Processor.LineCounter;
using TopCoder.File.Statistics.Tests;

namespace TopCoder.File.Statistics.Reporting.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="XmlStatsReporting"/> class.
    /// </summary>
    [TestFixture]
    public class TestXmlStatsReporting : TestHelper
    {
        /// <summary>
        /// A sample Metric name.
        /// </summary>
        private const string name = "Word count";

        /// <summary>
        /// A sample Metric display name.
        /// </summary>
        private const string displayName = "wcount";

        /// <summary>
        /// A sample Metric description.
        /// </summary>
        private const string description = "Count the number of words";

        /// <summary>
        /// The processor alias name.
        /// We'll test with 2 processor alias, composed of alias + "1"/"2".
        /// See SetUp().
        /// </summary>
        private const string alias = "Processor alias";

        /// <summary>
        /// A sample initial metric to be stored in StatsResult.
        /// Initialized in SetUp().
        /// </summary>
        private Metric metric;

        /// <summary>
        /// A sample statistic result for a file.
        /// Initialized in SetUp().
        /// </summary>
        private StatsResult result;

        /// <summary>
        /// The IFileProcessor which generates the result.
        /// Only the alias name is important for the report.
        /// We'll create 2 processor instances with the name alias + "1"/"2".
        /// Initialized to TextLineCounter() in SetUp().
        /// </summary>
        private IFileProcessor[] processor;

        /// <summary>
        /// The <see cref="XmlStatsReporting"/> instance to test.
        /// Initialized in SetUp() with 1 sample result.
        /// </summary>
        private IStatsReporting report;

        /// <summary>
        /// Initializes the <see cref="XmlStatsReporting"/> instance to test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            // load test_files/Reporting_xml.xml
            // used for constructor with nameSpace and prefix argument
            base.LoadConfigFile(@"..\..\test_files\Reporting_xml.xml");

            // initialize the IFileProcessor
            // only the alias name is important
            processor = new IFileProcessor[]
                {
                    new TextLineCounter(alias + "1", new AnyFileMatcher()),
                    new TextLineCounter(alias + "2", new AnyFileMatcher())
                };

            // create a StatsResult containing a metric with an initial value of 10
            metric = new Metric(name, displayName, description);
            result = new StatsResult();
            result[metric] = 10;

            // initialize the XmlStatsReporting
            // reset to test_files folder
            // and aggregate a result
            report = new XmlStatsReporting();
            report.Reset(new DirectoryInfo(@"..\..\test_files"));
            report.Aggregate(processor[0], new FileInfo(SampleText), result);
        }

        /// <summary>
        /// Tests StylesheetPropertyKey.
        /// </summary>
        [Test]
        public void TestStylesheetPropertyKey()
        {
            Assert.AreEqual("stylesheet", XmlStatsReporting.StylesheetPropertyKey,
                "The property key name is wrong");
        }

        /// <summary>
        /// Tests GenerateReport for raw xml data.
        /// </summary>
        [Test]
        public void TestGenerateReport()
        {
            // generate the report and load the expected result
            string rawXml = report.GenerateReport();
            string expectedXml = ReadContent(@"..\..\test_files\Reporting_Result_Simple.xml", false);

            // the last line may not have NewLine, so we won't compare the newline
            Assert.AreEqual(expectedXml.Replace(Environment.NewLine, ""),
                rawXml.Replace(Environment.NewLine, ""),
                "The pretty raw xml format is not accurate");
        }

        /// <summary>
        /// Tests GenerateReport given another statistic
        /// with different metric.
        /// </summary>
        [Test]
        public void TestGenerateReportAdd()
        {
            // create a new Metric and add to existing result
            result[new Metric(name + "1", displayName + "1", description + "1")] = 7;
            // aggregate for a new file with processor[1]
            report.Aggregate(processor[1], new FileInfo(@"..\..\test_files\test\aaa\sample.txt"), result);

            // generate the report and load the expected result
            string rawXml = report.GenerateReport();
            string expectedXml = ReadContent(@"..\..\test_files\Reporting_Result_Two.xml", false);

            // the last line may not have NewLine, so we won't compare the newline
            Assert.AreEqual(expectedXml.Replace(Environment.NewLine, ""),
                rawXml.Replace(Environment.NewLine, ""),
                "The pretty raw xml format is not accurate");
        }

        /// <summary>
        /// Tests GenerateReport with a stylesheet.
        /// </summary>
        [Test]
        public void TestGenerateReportXslt()
        {
            // initialize report with stylesheet
            report = new XmlStatsReporting(new FileInfo(@"..\..\test_files\Reporting_Stylesheet.xsl"));
            report.Reset(new DirectoryInfo(@"..\..\test_files"));
            report.Aggregate(processor[0], new FileInfo(SampleText), result);

            // generate the report and load the expected result
            string htmlResult = report.GenerateReport();
            string expectedXml = ReadContent(@"..\..\test_files\Reporting_Stylesheet_Result.html", false);

            // the last line may not have NewLine, so we won't compare the newline
            Assert.AreEqual(expectedXml.Replace(Environment.NewLine, ""),
                htmlResult.Replace(Environment.NewLine, ""),
                "The tranformed result is not accurate");
        }

        /// <summary>
        /// Tests constructor(FileInfo) throws ArgumentNullException
        /// given null argument.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructorFileInfoNull()
        {
            new XmlStatsReporting(null);
        }

        /// <summary>
        /// Tests constructor(FileInfo) throws ArgumentException
        /// given a non-existing xsl file.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructorFileInfoInvalid()
        {
            new XmlStatsReporting(new FileInfo("notexists"));
        }

        /// <summary>
        /// Tests constructor(string, string) with a stylesheet defined.
        /// </summary>
        [Test]
        public void TestConstructorStylesheet()
        {
            // use stylesheet property
            report = new XmlStatsReporting(ValidNamespace, "reporting.");
            report.Reset(new DirectoryInfo(@"..\..\test_files"));
            report.Aggregate(processor[0], new FileInfo(SampleText), result);

            // generate the report and load the expected result
            string htmlResult = report.GenerateReport();
            string expectedXml = ReadContent(@"..\..\test_files\Reporting_Stylesheet_Result.html", false);

            // the last line may not have NewLine, so we won't compare the newline
            Assert.AreEqual(expectedXml.Replace(Environment.NewLine, ""),
                htmlResult.Replace(Environment.NewLine, ""),
                "The tranformed result is not accurate");
        }

        /// <summary>
        /// Tests constructor(string, string) with a stylesheet defined
        /// and a more complex statistic.
        /// </summary>
        [Test]
        public void TestConstructorStylesheetComplex()
        {
            // use stylesheet property
            report = new XmlStatsReporting(ValidNamespace, "reporting.");
            report.Reset(new DirectoryInfo(@"..\..\test_files"));
            // aggregate two stats
            report.Aggregate(processor[0], new FileInfo(SampleText), result);

            // create a new Metric and add to existing result
            result[new Metric(name + "1", displayName + "1", description + "1")] = 7;
            // aggregate for a new file with processor[1]
            report.Aggregate(processor[1], new FileInfo(@"..\..\test_files\test\aaa\sample.txt"), result);

            // generate the report and load the expected result
            string htmlResult = report.GenerateReport();
            string expectedXml = ReadContent(@"..\..\test_files\Reporting_Stylesheet_Result_Two.html", false);

            // the last line may not have NewLine, so we won't compare the newline
            Assert.AreEqual(expectedXml.Replace(Environment.NewLine, ""),
                htmlResult.Replace(Environment.NewLine, ""),
                "The tranformed result is not accurate");
        }

        /// <summary>
        /// Tests constructor(string, string) doesn't use any stylesheet
        /// given non-existing prefix.
        /// </summary>
        [Test]
        public void TestConstructor3NotExist()
        {
            // no stylesheet property defined
            report = new XmlStatsReporting(ValidNamespace, "foobar.");
            report.Reset(new DirectoryInfo(@"..\..\test_files"));
            report.Aggregate(processor[0], new FileInfo(SampleText), result);

            // generate the report (raw xml is returned) and load the expected result
            string rawXml = report.GenerateReport();
            string expectedXml = ReadContent(@"..\..\test_files\Reporting_Result_Simple.xml", false);

            // the last line may not have NewLine, so we won't compare the newline
            Assert.AreEqual(expectedXml.Replace(Environment.NewLine, ""),
                rawXml.Replace(Environment.NewLine, ""),
                "The pretty raw xml format is not accurate");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentNullException
        /// given null nameSpace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor3NullNamespace()
        {
            new XmlStatsReporting(null, "reporting.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentNullException
        /// given null prefix.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor3NullPrefix()
        {
            new XmlStatsReporting(ValidNamespace, null);
        }

        /// <summary>
        /// Tests constructor(string, string) throws ArgumentException
        /// given empty namespace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor3EmptyNamespace()
        {
            new XmlStatsReporting(" ", "reporting.");
        }

        /// <summary>
        /// Tests constructor(string, string) throws ConfigurationException
        /// given a non-existing stylesheet.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructor3FileNotExist()
        {
            new XmlStatsReporting(ValidNamespace, "notexist.");
        }
        
        /// <summary>
        /// Tests ProposeOptions() return a single ArgumentOption.
        /// </summary>
        [Test]
        public void TestProposeOptions()
        {
            IList options = ((IArgumentHandler) report).ProposeOptions();

            // a single ArgumentOption
            Assert.AreEqual(1, options.Count, "1 element is returned");
            Assert.IsTrue(options[0] is ArgumentOption, "ArgumentOption is returned");
            
            ArgumentOption option = (ArgumentOption) options[0];
            Assert.AreEqual(XmlStatsReporting.StylesheetPropertyKey, option.SwitchName,
                "The switch name is incorrect");
            Assert.IsTrue(option.HelpMessage.Length > 0, "There should be a help message");
            Assert.IsFalse(option.IsRequired, "This switch is optional");
            Assert.IsNull(option.Validator, "No validator for this switch");
        }
        
        /// <summary>
        /// Tests HandleArguments() with empty value does nothing
        /// and the current stylesheet is preserved.
        /// </summary>
        [Test]
        public void TestHandleArguments()
        {
            // initialize report with stylesheet
            report = new XmlStatsReporting(new FileInfo(@"..\..\test_files\Reporting_Stylesheet.xsl"));
            report.Reset(new DirectoryInfo(@"..\..\test_files"));
            report.Aggregate(processor[0], new FileInfo(SampleText), result);

            // empty dictionary, this should not reset the stylesheet
            ((IArgumentHandler) report).HandleArguments(new Hashtable());
            
            // generate the report and load the expected result
            string htmlResult = report.GenerateReport();
            string expectedXml = ReadContent(@"..\..\test_files\Reporting_Stylesheet_Result.html", false);

            // the last line may not have NewLine, so we won't compare the newline
            Assert.AreEqual(expectedXml.Replace(Environment.NewLine, ""),
                htmlResult.Replace(Environment.NewLine, ""),
                "The tranformed result is not accurate");
        }
        
        /// <summary>
        /// Tests HandleArguments() with stylesheet switch with some value.
        /// </summary>
        [Test]
        public void TestHandleArgumentsValid()
        {
            IDictionary args = new Hashtable();
            args.Add(XmlStatsReporting.StylesheetPropertyKey, @"..\..\test_files\Reporting_Stylesheet.xsl");

            // the report has no stylesheet initially
            ((IArgumentHandler) report).HandleArguments(args);
            
            // generate the report and load the expected result
            string htmlResult = report.GenerateReport();
            string expectedXml = ReadContent(@"..\..\test_files\Reporting_Stylesheet_Result.html", false);

            // the last line may not have NewLine, so we won't compare the newline
            Assert.AreEqual(expectedXml.Replace(Environment.NewLine, ""),
                htmlResult.Replace(Environment.NewLine, ""),
                "The tranformed result is not accurate");
        }
        
        /// <summary>
        /// Tests HandleArguments() throws ArgumentNullException
        /// given null argument.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestHandleArgumentsNull()
        {
            ((IArgumentHandler) report).HandleArguments(null);
        }
        
        /// <summary>
        /// Tests HandleArguments() throws ArgumentException
        /// given non-string value.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestHandleArgumentsNonString()
        {
            IDictionary args = new Hashtable();
            args.Add(XmlStatsReporting.StylesheetPropertyKey, 123);
            ((IArgumentHandler) report).HandleArguments(args);
        }
        
        /// <summary>
        /// Tests HandleArguments() throws InvalidArgumentException
        /// given a non-existing file.
        /// </summary>
        [Test]
        [ExpectedException(typeof(InvalidArgumentException))]
        public void TestHandleArgumentsNonExist()
        {
            IDictionary args = new Hashtable();
            args.Add(XmlStatsReporting.StylesheetPropertyKey, @"notexists.xsl");
            ((IArgumentHandler) report).HandleArguments(args);
        }
    }
}
