/*
 * TestArgumentOption.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using TopCoder.Util.DataValidator;
using NUnit.Framework;

namespace TopCoder.File.Statistics.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="ArgumentOption"/> class.
    /// </summary>
    [TestFixture]
    public class TestArgumentOption
    {
        /// <summary>
        /// A sample switch name.
        /// </summary>
        private const string name = "The switch name";

        /// <summary>
        /// A sample help message for the switch.
        /// </summary>
        private const string help = "This is the switch help message";

        /// <summary>
        /// A sample validator to use.
        /// </summary>
        private IValidator validator = new EqualsValidator(123);

        /// <summary>
        /// A <see cref="ArgumentOption"/> instance to test.
        /// Initialized in SetUp().
        /// </summary>
        private ArgumentOption option;

        /// <summary>
        /// Initializes the <see cref="ArgumentOption"/> instance to test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            option = new ArgumentOption(name, help, true);
        }

        /// <summary>
        /// Tests property getters accuracy.
        /// </summary>
        [Test]
        public void TestGetters()
        {
            // assert property getters
            Assert.AreEqual(name, option.SwitchName, "The SwitchName is incorrect");
            Assert.AreEqual(help, option.HelpMessage, "The HelpMessage is incorrect");
            Assert.IsTrue(option.IsRequired, "The IsRequired flag is incorrect");
            Assert.IsNull(option.Validator, "No validator is specified");
        }

        /// <summary>
        /// Tests constructor with 3 arguments throws ArgumentNullException
        /// given null switch name.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructorNullSwitchName()
        {
            new ArgumentOption(null, help, true);
        }

        /// <summary>
        /// Tests constructor with 3 arguments throws ArgumentNullException
        /// given null help message.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructorNullHelp()
        {
            new ArgumentOption(name, null, true);
        }

        /// <summary>
        /// Tests constructor with 3 arguments throws ArgumentException
        /// given empty switch name.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructorEmptyName()
        {
            new ArgumentOption(" ", help, true);
        }

        /// <summary>
        /// Tests constructor with 3 arguments throws ArgumentException
        /// given empty help message.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructorEmptyHelp()
        {
            new ArgumentOption(name, " ", true);
        }

        /// <summary>
        /// Tests constructor with 4 arguments accuracy.
        /// </summary>
        [Test]
        public void TestConstructor2()
        {
            option = new ArgumentOption(name, help, false, validator);

            // assert property getters
            Assert.AreEqual(name, option.SwitchName, "The SwitchName is incorrect");
            Assert.AreEqual(help, option.HelpMessage, "The HelpMessage is incorrect");
            Assert.IsFalse(option.IsRequired, "The IsRequired flag is incorrect");
            Assert.AreSame(validator, option.Validator, "The validator is incorrect");
        }

        /// <summary>
        /// Tests constructor with 4 arguments throws ArgumentNullException
        /// given null switch name.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor2NullSwitchName()
        {
            new ArgumentOption(null, help, false, validator);
        }

        /// <summary>
        /// Tests constructor with 4 arguments throws ArgumentNullException
        /// given null help message.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor2NullHelpMessage()
        {
            new ArgumentOption(name, null, false, validator);
        }

        /// <summary>
        /// Tests constructor with 4 arguments throws ArgumentNullException
        /// given null validator.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructor2NullValidator()
        {
            new ArgumentOption(name, help, false, null);
        }

        /// <summary>
        /// Tests constructor with 4 arguments throws ArgumentException
        /// given empty switch name.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor2EmptySwitchName()
        {
            new ArgumentOption(" ", help, false, validator);
        }

        /// <summary>
        /// Tests constructor with 4 arguments throws ArgumentException
        /// given empty help message.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructor2EmptyHelpMessage()
        {
            new ArgumentOption(name, " ", false, validator);
        }
    }
}
