/*
 * TestFileStatistics.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.Collections;
using System.IO;
using NUnit.Framework;
using TopCoder.File.Statistics.Matcher;
using TopCoder.File.Statistics.Processor.LineCounter;
using TopCoder.File.Statistics.Reporting;
using TopCoder.File.Statistics.Tests;

namespace TopCoder.File.Statistics.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="FileStatistics"/> class.
    /// </summary>
    [TestFixture]
    public class TestFileStatistics : TestHelper
    {
        /// <summary>
        /// The <see cref="FileStatistics"/> instance to test.
        /// Initialized in SetUp().
        /// </summary>
        private FileStatistics statistics;

        /// <summary>
        /// Initializes the <see cref="FileStatistics"/> instance to test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            // load test_files/FileStatistics.xml
            base.LoadConfigFile(ValidConfig);

            // initialize FileStatistics
            statistics = new FileStatistics(ValidNamespace);
        }

        /// <summary>
        /// Tests the static properties.
        /// </summary>
        [Test]
        public void TestStaticProperties()
        {
            Assert.AreEqual("processors", FileStatistics.ProcessorsPropertyKey,
                "The processors property key is wrong");
            Assert.AreEqual("reporting", FileStatistics.ReportingPropertyKey,
                "The reporting property key is wrong");
            Assert.AreEqual("matchall", FileStatistics.MatchAllPropertyKey,
                "The matchall property key is wrong");
            Assert.AreEqual("classname", FileStatistics.ClassnamePropertyKey,
                "The classname property key is wrong");
        }

        /// <summary>
        /// Tests constructor(string) loads processors correctly
        /// and AllFileProcessors getter is correct.
        /// </summary>
        [Test]
        public void TestConstructorProcessor()
        {
            // 2 processors
            IList processors = statistics.AllFileProcessors;
            Assert.AreEqual(2, processors.Count, "2 processors are registered");

            // 1st processor
            IFileProcessor processor = processors[0] as IFileProcessor;
            Assert.AreEqual("Text File", processor.Alias,
                "The 1st processor alias is Text File");
            Assert.IsTrue(processor is TextLineCounter,
                "The 1st processor is TextLineCounter");
            Assert.IsTrue(processor.FileMatcher is ExtensionMatcher,
                "The 1st matcher is ExtensionMatcher");
            // assert the matcher can match only txt and sql
            Assert.IsTrue(processor.FileMatcher.Match(new FileInfo("test.txt")),
                "txt should match");
            Assert.IsTrue(processor.FileMatcher.Match(new FileInfo("test.sql")),
                "sql should match");
            Assert.IsFalse(processor.FileMatcher.Match(new FileInfo("test.bin")),
                "bin should not match");

            // 2nd processor
            processor = processors[1] as IFileProcessor;
            Assert.AreEqual("Source File", processor.Alias,
                "The 2nd processor alias is Source File");
            Assert.IsTrue(processor is CStyleLineCounter,
                "The 2nd processor is CStyleLineCounter");
            Assert.IsTrue(processor.FileMatcher is RegexMatcher,
                "The 2nd matcher is RegexMatcher");
            // assert the matcher can match only .c
            Assert.IsFalse(processor.FileMatcher.Match(new FileInfo("test.txt")),
                "txt should not match");
            Assert.IsFalse(processor.FileMatcher.Match(new FileInfo("test.sql")),
                "sql should not match");
            Assert.IsTrue(processor.FileMatcher.Match(new FileInfo("test.c")),
                "c should match");
        }

        /// <summary>
        /// Tests constructor(string) loads reporting instance correctly
        /// and Reporting getter/setter is correct.
        /// </summary>
        [Test]
        public void TestConstructorReporting()
        {
            // XmlStatsReporting
            IStatsReporting report = statistics.Reporting;
            Assert.IsTrue(report is XmlStatsReporting,
                "The XmlStatsReporting instance is used");
            // make sure stylesheet is loaded
            Assert.IsFalse(report.GenerateReport().StartsWith("<?xml"),
                "The report should not be in xml format");

            // set to another instance
            report = new BasicStatsReporting();
            statistics.Reporting = report;
            Assert.IsTrue(statistics.Reporting is BasicStatsReporting,
                "The BasicStatsReporting instance is now used");
            Assert.IsFalse(report.GenerateReport().StartsWith("<?xml"),
                "The report should not be in xml format");
        }

        /// <summary>
        /// Tests constructor(string) loads matchall with true given "on".
        /// </summary>
        [Test]
        public void TestConstructorMatchAllOn()
        {
            Assert.IsTrue(statistics.IsMatchAll, "The MatchAll property is on");
        }

        /// <summary>
        /// Tests constructor(string) loads matchall with true given "TruE".
        /// </summary>
        [Test]
        public void TestConstructorMatchAllTrue()
        {
            base.LoadConfigFile(@"..\..\test_files\FileStatistics_MatchAll_True.xml");

            statistics = new FileStatistics(ValidNamespace);
            Assert.IsTrue(statistics.IsMatchAll, "The MatchAll property is TruE");
        }

        /// <summary>
        /// Tests constructor(string) loads matchall with true given "yEs".
        /// </summary>
        [Test]
        public void TestConstructorMatchAllYes()
        {
            base.LoadConfigFile(@"..\..\test_files\FileStatistics_MatchAll_Yes.xml");

            statistics = new FileStatistics(ValidNamespace);
            Assert.IsTrue(statistics.IsMatchAll, "The MatchAll property is yEs");
        }

        /// <summary>
        /// Tests constructor(string) loads matchall with false
        /// if the property is not specified.
        /// </summary>
        [Test]
        public void TestConstructorMatchAllDefault()
        {
            base.LoadConfigFile(@"..\..\test_files\FileStatistics_MatchAll_Default.xml");

            statistics = new FileStatistics(ValidNamespace);
            Assert.IsFalse(statistics.IsMatchAll,
                "The MatchAll property is missing and should default to false");
        }

        /// <summary>
        /// Tests constructor(string) loads AnyFileMatcher
        /// if no matcher is specified.
        /// </summary>
        [Test]
        public void TestConstructorAnyFileMatcher()
        {
            // load test_files/FileStatistics_NoMatcher.xml
            base.LoadConfigFile(@"..\..\test_files\FileStatistics_NoMatcher.xml");

            // initialize FileStatistics
            statistics = new FileStatistics(ValidNamespace);

            // 1 processor
            IList processors = statistics.AllFileProcessors;
            Assert.AreEqual(1, processors.Count, "1 processor is registered");

            // assert processor
            IFileProcessor processor = processors[0] as IFileProcessor;
            Assert.AreEqual("Text File", processor.Alias,
                "The processor alias is Text File");
            Assert.IsTrue(processor is TextLineCounter,
                "The processor is TextLineCounter");
            Assert.IsTrue(processor.FileMatcher is AnyFileMatcher,
                "The matcher is AnyFileMatcher");
        }

        /// <summary>
        /// Tests constructor(string) throws ConfigurationException
        /// if no processors.names is specified.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructorNoProcessorsNames()
        {
            // load test_files/FileStatistics_Invalid_NoProcessorsNames.xml");
            base.LoadConfigFile(@"..\..\test_files\FileStatistics_Invalid_NoProcessorsNames.xml");

            // initialize FileStatistics
            new FileStatistics(ValidNamespace);
        }

        /// <summary>
        /// Tests constructor(string) throws ConfigurationException
        /// if the processor aliasname is missing.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructorNoProcessorsAlias()
        {
            // load test_files/FileStatistics_Invalid_NoProcessorAlias.xml
            base.LoadConfigFile(@"..\..\test_files\FileStatistics_Invalid_NoProcessorAlias.xml");

            // initialize FileStatistics
            new FileStatistics(ValidNamespace);
        }

        /// <summary>
        /// Tests constructor(string) throws ConfigurationException
        /// if the processor.classname is not instance of IFileProcessor.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructorInvalidProcessorsClassname()
        {
            // load test_files/FileStatistics_Invalid_InvalidProcessor.xml
            base.LoadConfigFile(@"..\..\test_files\FileStatistics_Invalid_InvalidProcessor.xml");

            // initialize FileStatistics
            new FileStatistics(ValidNamespace);
        }

        /// <summary>
        /// Tests constructor(string) throws ConfigurationException
        /// if no reporting.classname is specified.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructorNoReporting()
        {
            // load test_files/FileStatistics_Invalid_NoReporting.xml
            base.LoadConfigFile(@"..\..\test_files\FileStatistics_Invalid_NoReporting.xml");

            // initialize FileStatistics
            new FileStatistics(ValidNamespace);
        }

        /// <summary>
        /// Tests constructor(string) throws ConfigurationException
        /// if reporting.classname is not instance of IStatsReporting.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ConfigurationException))]
        public void TestConstructorInvalidReporting()
        {
            // load test_files/FileStatistics_Invalid_InvalidReporting.xml
            base.LoadConfigFile(@"..\..\test_files\FileStatistics_Invalid_InvalidReporting.xml");

            // initialize FileStatistics
            new FileStatistics(ValidNamespace);
        }

        /// <summary>
        /// Tests constructor(string) throws ArgumentNullException
        /// given null namespace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructorNullNamespace()
        {
            new FileStatistics(null);
        }

        /// <summary>
        /// Tests constructor(string) throws ArgumentException
        /// given empty namespace.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructorEmptyNamespace()
        {
            new FileStatistics(" ");
        }

        /// <summary>
        /// Tests constructor() accuracy.
        /// </summary>
        [Test]
        public void TestConstructorDefault()
        {
            // use default constructor
            statistics = new FileStatistics();

            Assert.AreEqual(0, statistics.AllFileProcessors.Count, "No processor is registered");
            Assert.IsFalse(statistics.IsMatchAll, "MatchAll default to false");
            Assert.IsTrue(statistics.Reporting is XmlStatsReporting,
                "The default reporting is loaded");

            // the default reporting doesn't use stylesheet
            Assert.IsTrue(statistics.Reporting.GenerateReport().StartsWith("<?xml"),
                "The default reporting is xml");
        }

        /// <summary>
        /// Tests Remove() by removing 1st processor.
        /// </summary>
        [Test]
        public void TestRemove()
        {
            // remove 1st processor "Text File"
            IFileProcessor old = statistics.RemoveFileProcessor("Text File");

            // assert return value
            Assert.AreEqual("Text File", old.Alias, "Text File is removed");

            // we have 1 processor
            IList processors = statistics.AllFileProcessors;
            Assert.AreEqual(1, processors.Count, "1 processor left");

            // assert the processor
            Assert.AreEqual("Source File", ((IFileProcessor) processors[0]).Alias,
                "The processor is wrong");
        }

        /// <summary>
        /// Tests Remove() by removing non-existing processor.
        /// </summary>
        [Test]
        public void TestRemoveNotExist()
        {
            // remove some non-existing processor
            IFileProcessor old = statistics.RemoveFileProcessor("Not Exists");

            // assert return value
            Assert.IsNull(old, "No processor is removed");

            // we still have 2 processors
            IList processors = statistics.AllFileProcessors;
            Assert.AreEqual(2, processors.Count, "2 processor still there");

            // assert the processor
            Assert.AreEqual("Text File", ((IFileProcessor) processors[0]).Alias,
                "The 1st processor is wrong");
            Assert.AreEqual("Source File", ((IFileProcessor) processors[1]).Alias,
                "The 2nd processor is wrong");
        }

        /// <summary>
        /// Tests Remove() throws ArgumentNullException
        /// given null argument.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestRemoveNull()
        {
            statistics.RemoveFileProcessor(null);
        }

        /// <summary>
        /// Tests Remove() throws ArgumentException
        /// given empty argument.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestRemoveEmpty()
        {
            statistics.RemoveFileProcessor(" ");
        }

        /// <summary>
        /// Tests Add() by adding a new processor.
        /// </summary>
        [Test]
        public void TestAdd()
        {
            // add a new TextLineCounter
            IFileProcessor newProcessor = new TextLineCounter("New processor", new AnyFileMatcher());
            statistics.AddFileProcessor(newProcessor);

            // we have 3 processors now
            IList processors = statistics.AllFileProcessors;
            Assert.AreEqual(3, processors.Count, "3 processors are registered");

            // assert the processor order
            Assert.AreEqual("Text File", ((IFileProcessor) processors[0]).Alias,
                "The 1st processor is wrong");
            Assert.AreEqual("Source File", ((IFileProcessor) processors[1]).Alias,
                "The 2nd processor is wrong");
            Assert.AreEqual("New processor", ((IFileProcessor) processors[2]).Alias,
                "The 3rd processor is wrong");
        }

        /// <summary>
        /// Tests Add() by replacing the 1st processor.
        /// </summary>
        [Test]
        public void TestAddReplace()
        {
            // add a new CStyleLineCounter to replace the 1st alias "Text File"
            // note that the 1st one was originally a TextLineCounter
            IFileProcessor newProcessor = new CStyleLineCounter("Text File", new AnyFileMatcher());
            statistics.AddFileProcessor(newProcessor);

            // we have 2 processors
            IList processors = statistics.AllFileProcessors;
            Assert.AreEqual(2, processors.Count, "2 processors are registered");

            // assert the processor order
            Assert.AreEqual("Source File", ((IFileProcessor) processors[0]).Alias,
                "The 2nd processor becomes the 1st processor");
            Assert.AreEqual("Text File", ((IFileProcessor) processors[1]).Alias,
                "The 1st processor is replaced and becomes 2nd");
            Assert.AreSame(newProcessor, processors[1],
                "The new processor is at the end of the list");
            Assert.IsTrue(processors[0] is CStyleLineCounter,
                "The 1st processor is now CStyleListCounter");
            Assert.IsTrue(processors[1] is CStyleLineCounter,
                "The 2nd processor is now also CStyleLineCounter");
        }

        /// <summary>
        /// Tests Add() throws ArgumentNullException
        /// given null argument.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestAddNull()
        {
            statistics.AddFileProcessor(null);
        }

        /// <summary>
        /// Tests Clear().
        /// </summary>
        [Test]
        public void TestClear()
        {
            statistics.ClearAllFileProcessors();

            // no more processor
            Assert.AreEqual(0, statistics.AllFileProcessors.Count, "All processors cleared");
        }

        /// <summary>
        /// Tests indexer accuracy.
        /// </summary>
        [Test]
        public void TestIndexer()
        {
            // 1st processor
            IFileProcessor processor = statistics["Text File"];
            Assert.AreEqual("Text File", processor.Alias, "The 1st processor alias is retrieved");
            Assert.IsTrue(processor is TextLineCounter, "1st processor is TextLineCounter");

            // 2nd processor
            processor = statistics["Source File"];
            Assert.AreEqual("Source File", processor.Alias, "The 2nd processor alias is retrieved");
            Assert.IsTrue(processor is CStyleLineCounter, "2nd processor is CStyleLineCounter");
        }

        /// <summary>
        /// Tests indexer returns null if alias cannot be found.
        /// </summary>
        [Test]
        public void TestIndexerNotFound()
        {
            // 1st processor
            Assert.IsNull(statistics["text file"], "Alias is case-sensitive");
            Assert.IsNull(statistics["notexists"], "Alias doesn't exist");
        }

        /// <summary>
        /// Tests indexer throws ArgumentNullException
        /// given null alias.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestIndexerNull()
        {
            IFileProcessor processor = statistics[null];
        }

        /// <summary>
        /// Tests indexer throws ArgumentException
        /// given empty alias.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestIndexerEmpty()
        {
            IFileProcessor processor = statistics[" "];
        }

        /// <summary>
        /// Tests ProcessFile() with the sample text file.
        /// </summary>
        [Test]
        public void TestProcessFileText()
        {
            SetXmlReporting();

            // process 1 text file (text processor is the 1st match)
            statistics.ProcessFile(new FileInfo(SampleText));

            string report = statistics.Reporting.GenerateReport();

            // assert the report
            AssertXml(report, "/report/overall/line", "4");
            AssertXml(report, "/report/files/file[@name=\"test.txt\"]/line", "4");
            AssertXml(report, "/report/directories/directory[@name=\"test\"]/line", null);
            AssertXml(report, "/report/aliases/alias[@name=\"Text File\"]/line", "4");
        }

        /// <summary>
        /// Tests ProcessFile() with the sample C file.
        /// </summary>
        [Test]
        public void TestProcessFileSource()
        {
            SetXmlReporting();

            // process 1 C file (CStyle processor is the 1st match)
            statistics.ProcessFile(new FileInfo(@"..\..\test_files\test\sample.c"));

            string report = statistics.Reporting.GenerateReport();

            // assert the report
            AssertXml(report, "/report/overall/line", "22");
            AssertXml(report, "/report/files/file[@name=\"sample.c\"]/line", "22");
            AssertXml(report, "/report/directories/directory[@name=\"test\"]/line", null);
            AssertXml(report, "/report/aliases/alias[@name=\"Source File\"]/line", "22");
        }

        /// <summary>
        /// Tests ProcessFile() with no processor.
        /// </summary>
        [Test]
        public void TestProcessFileTextNoProcessor()
        {
            SetXmlReporting();

            // remove the text processor
            statistics.RemoveFileProcessor("Text File");

            // process 1 text file (no processor match)
            statistics.ProcessFile(new FileInfo(SampleText));

            string report = statistics.Reporting.GenerateReport();

            // assert the report
            AssertXml(report, "/report/overall/line", null);
            AssertXml(report, "/report/files/file[@name=\"test.txt\"]/line", null);
            AssertXml(report, "/report/directories/directory[@name=\"test\"]/line", null);
            AssertXml(report, "/report/aliases/alias[@name=\"Text File\"]/line", null);
        }

        /// <summary>
        /// Tests ProcessFile() that matches both processors.
        /// </summary>
        [Test]
        public void TestProcessFileTextBothProcessorsAllMatch()
        {
            SetXmlReporting();

            // change the second processor to text line counter which matches any file
            IFileProcessor newProcessor = new TextLineCounter("Source File", new AnyFileMatcher());
            statistics.AddFileProcessor(newProcessor);

            // process 1 text file (both processors match and processed matchall = true)
            Assert.IsTrue(statistics.IsMatchAll, "With MatchAll = true");
            statistics.ProcessFile(new FileInfo(SampleText));

            string report = statistics.Reporting.GenerateReport();

            // assert the report
            AssertXml(report, "/report/overall/line", "8"); // both processors aggregated
            AssertXml(report, "/report/files/file[@name=\"test.txt\"]/line", "8");
            AssertXml(report, "/report/directories/directory[@name=\"test\"]/line", null);
            AssertXml(report, "/report/aliases/alias[@name=\"Text File\"]/line", "4");
            AssertXml(report, "/report/aliases/alias[@name=\"Source File\"]/line", "4");
        }

        /// <summary>
        /// Tests ProcessFile() that matches 1st processor only
        /// although both would match because we have matchall = false.
        /// </summary>
        [Test]
        public void TestProcessFileTextBothProcessorsFirstMatch()
        {
            SetXmlReporting();

            // change the second processor to text line counter which matches any file
            IFileProcessor newProcessor = new TextLineCounter("Source File", new AnyFileMatcher());
            statistics.AddFileProcessor(newProcessor);

            // process 1 text file (both processors match but only 1st one runs matchall = false)
            statistics.IsMatchAll = false;
            statistics.ProcessFile(new FileInfo(SampleText));

            string report = statistics.Reporting.GenerateReport();

            // assert the report
            AssertXml(report, "/report/overall/line", "4"); // both processors aggregated
            AssertXml(report, "/report/files/file[@name=\"test.txt\"]/line", "4");
            AssertXml(report, "/report/directories/directory[@name=\"test\"]/line", null);
            AssertXml(report, "/report/aliases/alias[@name=\"Text File\"]/line", "4");

            // Source File is not run because Text File matches first
            AssertXml(report, "/report/aliases/alias[@name=\"Source File\"]/line", null);
        }

        /// <summary>
        /// Tests ProcessFile() throws ArgumentNullException
        /// given null argument.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestProcessFileNull()
        {
            statistics.ProcessFile(null);
        }

        /// <summary>
        /// Tests ProcessFile() throws IOException
        /// given file that doesn't exist.
        /// </summary>
        [Test]
        [ExpectedException(typeof(IOException))]
        public void TestProcessFileNotExist()
        {
            statistics.ProcessFile(new FileInfo("notexists"));
        }

        /// <summary>
        /// Tests ProcessFile() throws IOException
        /// given a directory.
        /// </summary>
        [Test]
        [ExpectedException(typeof(IOException))]
        public void TestProcessFileDirectory()
        {
            statistics.ProcessFile(new FileInfo(@"..\..\test_files"));
        }

        /// <summary>
        /// Tests GetAllArgumentHandlers() returns those instances
        /// that implement IArgumentHandler.
        /// </summary>
        public void TestGetAllArgumentHandlers()
        {
            IList args = statistics.GetAllArgumentHandlers();
            Assert.AreEqual(3, args.Count, "3 handlers implement IArgumentHandlers");

            // each offering 1 ArgumentOption
            // collect the switch name to be asserted
            ArrayList options = new ArrayList();
            foreach (IArgumentHandler arg in args)
            {
                IList opts = arg.ProposeOptions();
                Assert.AreEqual(1, opts.Count, "Each handler offer 1 option");
                options.Add(((ArgumentOption) opts[0]).SwitchName);
            }

            // assert the Argument Option switch name
            Assert.IsTrue(options.Contains(TextLineCounter.IgnoreBlankLinePropertyKey),
                "TextLineCounter implements IArgumentHandler");
            Assert.IsTrue(options.Contains(CStyleLineCounter.SupportAtLiteralPropertyKey),
                "CStyleLineCounter implements IArgumentHandler");
            Assert.IsTrue(options.Contains(XmlStatsReporting.StylesheetPropertyKey),
                "XmlStatsReporting implements IArgumentHandler");
        }

        /// <summary>
        /// Tests GetAllArgumentHandlers() with default file statistic constructor.
        /// </summary>
        public void TestGetAllArgumentHandlersEmpty()
        {
            statistics = new FileStatistics();
            IList args = statistics.GetAllArgumentHandlers();
            Assert.AreEqual(1, args.Count, "By default FileStatistics use XmlStatsReporting");
            IList opts = ((IArgumentHandler) args[0]).ProposeOptions();
            Assert.AreEqual(XmlStatsReporting.StylesheetPropertyKey, ((ArgumentOption) opts[0]).SwitchName);

            // set to basic text reporting which doesn't implement IArgumentHandler
            statistics.Reporting = new BasicStatsReporting();
            args = statistics.GetAllArgumentHandlers();
            Assert.AreEqual(0, args.Count, "No more IArgumentHandler");
        }

        /// <summary>
        /// Sets to plain xml reporting (no stylesheet)
        /// and set BaseDirectory to test_files/test
        /// </summary>
        private void SetXmlReporting()
        {
            statistics.Reporting = new XmlStatsReporting();
            statistics.Reporting.Reset(new DirectoryInfo(@"..\..\test_files\test"));
        }
    }
}
