/*
 * TestMetric.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.Collections;
using NUnit.Framework;

namespace TopCoder.File.Statistics.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="Metric"/> class.
    /// </summary>
    [TestFixture]
    public class TestMetric
    {
        /// <summary>
        /// A sample metric name.
        /// </summary>
        private const string name = "Word count";

        /// <summary>
        /// A sample metric display name.
        /// </summary>
        private const string displayName = "wcount";

        /// <summary>
        /// A sample metric description.
        /// </summary>
        private const string description = "Count the number of words";

        /// <summary>
        /// A <see cref="Metric"/> instance to test.
        /// Initialized in SetUp().
        /// </summary>
        private Metric metric;

        /// <summary>
        /// Initializes the <see cref="Metric"/> instance to test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            metric = new Metric(name, displayName, description);
        }

        /// <summary>
        /// Tests property getters accuracy.
        /// </summary>
        [Test]
        public void TestGetters()
        {
            // assert property getters
            Assert.AreEqual(name, metric.Name, "The metric name is incorrect");
            Assert.AreEqual(displayName, metric.DisplayName, "The metric display name is incorrect");
            Assert.AreEqual(description, metric.Description, "The metric description is incorrect");
        }

        /// <summary>
        /// Tests constructor throws ArgumentNullException
        /// given null name.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructorNullName()
        {
            new Metric(null, displayName, description);
        }

        /// <summary>
        /// Tests constructor throws ArgumentNullException
        /// given null display name.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructorNullDisplayName()
        {
            new Metric(name, null, description);
        }

        /// <summary>
        /// Tests constructor throws ArgumentNullException
        /// given null description.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructorNullDescription()
        {
            new Metric(name, displayName, null);
        }

        /// <summary>
        /// Tests constructor throws ArgumentException
        /// given empty name.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructorEmptyName()
        {
            new Metric(" ", displayName, description);
        }

        /// <summary>
        /// Tests constructor throws ArgumentException
        /// given empty display name.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructorEmptyDisplayName()
        {
            new Metric(name, " ", description);
        }

        /// <summary>
        /// Tests constructor throws ArgumentException
        /// given empty description.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestConstructorEmptyDescription()
        {
            new Metric(name, displayName, " ");
        }

        /// <summary>
        /// Tests Equals given Metrics with the same name,
        /// but different display name and description.
        /// </summary>
        [Test]
        public void TestEqualsTrue()
        {
            // create a new metric with the same name, but different display name, and description
            Metric m2 = new Metric(name, "whatever", "no description");
            Assert.IsTrue(metric.Equals(m2), "The metrics should be equal because the name is the same");
        }

        /// <summary>
        /// Tests Equals given different Metrics.
        /// </summary>
        [Test]
        public void TestEqualsFalse()
        {
            // create a new metric with different name, but the same display name, and description
            Metric m2 = new Metric(name + "1", displayName, description);
            Assert.IsFalse(metric.Equals(m2), "The metrics should not be equal");
        }

        /// <summary>
        /// Tests Equals given non Metric instance.
        /// </summary>
        [Test]
        public void TestEqualsNonMetric()
        {
            // they are definitely not equal
            Assert.IsFalse(metric.Equals("123"), "The objects are definitely different");
        }

        /// <summary>
        /// Tests Equals given null.
        /// </summary>
        [Test]
        public void TestEqualsNull()
        {
            // they are definitely not equal
            Assert.IsFalse(metric.Equals(null), "The objects are definitely different");
        }

        /// <summary>
        /// Tests GetHashCode.
        /// </summary>
        [Test]
        public void TestGetHashCode()
        {
            // GetHashCode is the name's hash code
            Assert.AreEqual(name.GetHashCode(), metric.GetHashCode(),
                "The metric hash code should be the name's hash code");
        }

        /// <summary>
        /// Tests Metric functionality as map key.
        /// </summary>
        [Test]
        public void TestMapKey()
        {
            // create a map
            // key: Metric[1-5]
            // value: [1-5]
            IDictionary map = new System.Collections.Hashtable();
            for (int i = 1; i <= 5; i++)
            {
                map.Add(new Metric(name + i, displayName + i, description + i), i);
            }

            // get back the map value and assert it
            for (int i = 1; i <= 5; i++)
            {
                // only name matters
                int val = (int) map[new Metric(name + i, "whatever", "N/A")];
                Assert.AreEqual(i, val, "The map value is not correctly retrieved");
            }
        }
    }
}
