/*
 * TestStatsResult.cs
 *
 * File Statistics
 * Version: 1.0
 *
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.Collections;
using NUnit.Framework;

namespace TopCoder.File.Statistics.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="StatsResult"/> class.
    /// </summary>
    [TestFixture]
    public class TestStatsResult
    {
        /// <summary>
        /// A sample StatsResult name.
        /// </summary>
        private const string name = "Word count";

        /// <summary>
        /// A sample StatsResult display name.
        /// </summary>
        private const string displayName = "wcount";

        /// <summary>
        /// A sample StatsResult description.
        /// </summary>
        private const string description = "Count the number of words";

        /// <summary>
        /// A sample initial metric to be stored in StatsResult.
        /// Initialized in SetUp().
        /// </summary>
        private Metric metric;

        /// <summary>
        /// A <see cref="StatsResult"/> instance to test.
        /// Initialized in SetUp().
        /// </summary>
        private StatsResult statsResult;

        /// <summary>
        /// Initializes the <see cref="StatsResult"/> instance to test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            statsResult = new StatsResult();

            // add a metric with an initial value of 10
            metric = new Metric(name, displayName, description);
            statsResult[metric] = 10;
        }

        /// <summary>
        /// Tests Metrics and this[Metric] property getter accuracy.
        /// </summary>
        [Test]
        public void TestMetrics()
        {
            // assert 1 metric
            IList metrics = statsResult.Metrics;
            Assert.AreEqual(1, metrics.Count, "1 metric is stored");

            // assert the metric
            Metric m = metrics[0] as Metric;
            Assert.AreEqual(metric, m, "The Metric is incorrect");
            Assert.AreEqual(10, statsResult[m], "The metric value should be 10");
        }

        /// <summary>
        /// Tests this[Metric] property setter by adding a new metric.
        /// </summary>
        [Test]
        public void TestMetricsAdd()
        {
            // add a new metric with a value of 20
            Metric newMetric = new Metric(name + "1", displayName, description);
            statsResult[newMetric] = 20;

            // assert 2 metrics
            IList metrics = statsResult.Metrics;
            Assert.AreEqual(2, metrics.Count, "2 metrics are stored now");

            // order is not guaranteed, so we need to determine which one is metric/newMetric
            int first = (((Metric) metrics[0]).Name == name) ? 0 : 1;

            // assert 1st metric
            Metric m = metrics[first] as Metric;
            Assert.AreEqual(metric, m, "The Metric is incorrect");
            Assert.AreEqual(10, statsResult[m], "The metric value should be 10");

            // assert newly added metric
            m = metrics[1 - first] as Metric;
            Assert.AreEqual(newMetric, m, "The new metric is incorrect");
            Assert.AreEqual(20, statsResult[m], "The new metric value should be 20");
        }

        /// <summary>
        /// Tests this[Metric] property setter replace the old value.
        /// </summary>
        [Test]
        public void TestMetricsReplace()
        {
            // replace the metric with a value of 30
            statsResult[metric] = 30;

            // assert 1 metric
            IList metrics = statsResult.Metrics;
            Assert.AreEqual(1, metrics.Count, "1 metric is updated");

            // assert the metric
            Metric m = metrics[0] as Metric;
            Assert.AreEqual(metric, m, "The Metric is incorrect");
            Assert.AreEqual(30, statsResult[m], "The metric value should be 30 now");
        }

        /// <summary>
        /// Tests this[Metric] property getter throws ArgumentNullException
        /// given null indexer.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestIndexerNullGet()
        {
            // get the value of null
            long val = statsResult[null];
        }

        /// <summary>
        /// Tests this[Metric] property setter throws ArgumentNullException
        /// given null indexer.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestIndexerNullSet()
        {
            // set the value of null
            statsResult[null] = 5;
        }

        /// <summary>
        /// Tests this[Metric] property getter throws ArgumentException
        /// given non-existing Metric.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestIndexerNotExistGet()
        {
            // get the value of some non-existing metric
            long val = statsResult[new Metric("noname", "whatever", "no description")];
        }

        /// <summary>
        /// Tests copy constructor accuracy.
        /// </summary>
        [Test]
        public void TestConstructorCopy()
        {
            // copy constructor
            StatsResult copy = new StatsResult(statsResult);

            // assert 1 metric is copied
            IList metrics = copy.Metrics;
            Assert.AreEqual(1, metrics.Count, "1 metric is copied");

            // assert the metric
            Metric m = metrics[0] as Metric;
            Assert.AreEqual(metric, m, "The Metric is incorrect");
            Assert.AreEqual(10, copy[m], "The metric value should be 10");
        }

        /// <summary>
        /// Tests copy constructor given a <see cref="StatsResult"/>
        /// containing 2 metrics.
        /// </summary>
        [Test]
        public void TestConstructorCopyTwo()
        {
            // add a new metric with a value of 15
            Metric newMetric = new Metric(name + "1", displayName, description);
            statsResult[newMetric] = 15;

            // copy constructor
            StatsResult copy = new StatsResult(statsResult);

            // assert 2 metrics are copied
            IList metrics = copy.Metrics;
            Assert.AreEqual(2, metrics.Count, "2 metrics are copied");

            // order is not guaranteed, so we need to determine which one is metric/newMetric
            int first = (((Metric) metrics[0]).Name == name) ? 0 : 1;

            // assert 1st metric
            Metric m = metrics[first] as Metric;
            Assert.AreEqual(metric, m, "The Metric is incorrect");
            Assert.AreEqual(10, statsResult[m], "The metric value should be 10");

            // assert newly added metric
            m = metrics[1 - first] as Metric;
            Assert.AreEqual(newMetric, m, "The new metric is incorrect");
            Assert.AreEqual(15, statsResult[m], "The new metric value should be 15");
        }

        /// <summary>
        /// Tests copy constructor throws ArgumentNullException
        /// given null argument.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestConstructorCopyNull()
        {
            new StatsResult(null);
        }

        /// <summary>
        /// Tests Aggregate by adding a value for an existing metric.
        /// </summary>
        [Test]
        public void TestAggregateAdd()
        {
            // create a new StatsResult containing metric with the same name
            Metric sameMetric = new Metric(name, displayName, description);
            StatsResult newResult = new StatsResult();
            newResult[sameMetric] = 7;

            // aggregate the new result
            statsResult.Aggregate(newResult);

            // assert 1 metric
            IList metrics = statsResult.Metrics;
            Assert.AreEqual(1, metrics.Count, "1 metric is aggregated");

            // assert the metric value is aggregated / added
            Metric m = metrics[0] as Metric;
            Assert.AreEqual(metric, m, "The Metric is incorrect");
            Assert.AreEqual(10 + 7, statsResult[m], "The metric value should be 17 now");
        }

        /// <summary>
        /// Tests Aggregate by adding a StatsResult
        /// containing 2 metrics (one already exists).
        /// </summary>
        [Test]
        public void TestAggregateAddTwo()
        {
            // create a new StatsResult containing metric with the same name
            Metric sameMetric = new Metric(name, displayName, description);
            StatsResult newResult = new StatsResult();
            newResult[sameMetric] = 3;

            // and a new metric with different name to the new StatsResult
            Metric newMetric = new Metric(name + "1", "no name", "no description");
            newResult[newMetric] = 2;

            // aggregate the new result
            statsResult.Aggregate(newResult);

            // assert 2 metrics
            IList metrics = statsResult.Metrics;
            Assert.AreEqual(2, metrics.Count, "2 metrics are aggregated");

            // order is not guaranteed, so we need to determine which one is metric/newMetric
            int first = (((Metric) metrics[0]).Name == name) ? 0 : 1;

            // assert 1st metric
            Metric m = metrics[first] as Metric;
            Assert.AreEqual(metric, m, "The Metric is incorrect");
            Assert.AreEqual(13, statsResult[m], "The metric value should be 13");

            // assert newly added metric
            m = metrics[1 - first] as Metric;
            Assert.AreEqual(newMetric, m, "The new metric is incorrect");
            Assert.AreEqual(2, statsResult[m], "The new metric value should be 2");
        }

        /// <summary>
        /// Tests Aggregate throws ArgumentNullException
        /// given null argument.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestAggregateNull()
        {
            statsResult.Aggregate(null);
        }
    }
}
